#ifndef _M3DMATH_H_
#define _M3DMATH_H_

#define PI    3.141592653589
#define PI180 0.017453292519

// 2-rozmerny vektor
struct tVec2 {
 float x,y;
};

// 3-rozmerny vektor
class tVec3 {
 public:
  struct {
   float x,y,z;
  };

  // konstruktory
  tVec3(float x=0.0f,float y=0.0f,float z=0.0f):x(x),y(y),z(z) {}
  tVec3(const tVec3 &vec):x(vec.x),y(vec.y),z(vec.z) {}
  tVec3(const tVec3 &a,const tVec3 &b):x(b.x-a.x),y(b.y-a.y),z(b.z-a.z) {}

  // scitanie vektorov
  const tVec3 operator+(const tVec3 &vec) const {
   return tVec3(x+vec.x,y+vec.y,z+vec.z);
  }
  // negacia kladneho vektora
  const tVec3 operator+() const {
   return tVec3(*this);
  }
  // pricitanie k vektoru
  const tVec3 &operator+=(const tVec3 &vec) {
   x+=vec.x; y+=vec.y; z+=vec.z;
   return *this;
  }
  // odcitanie vektorov
  const tVec3 operator-(const tVec3 &vec) const {
   return tVec3(x-vec.x,y-vec.y,z-vec.z);
  }
  // negacia zaporneho vektora
  const tVec3 operator-() const {
   return tVec3(-x,-y,-z);
  }
  // odcitanie od vektora
  const tVec3 &operator-=(const tVec3 &vec) {
   x-=vec.x; y-=vec.y; z-=vec.z;
   return *this;
  }
  // nasobenie skalarom
  const tVec3 &operator*=(const float s) {
   x*=s; y*=s; z*=s;
   return *this;
  }
  // delenie skalarom
  const tVec3 &operator/=(const float s) {
   const float recip=1.0f/s;
   x*=recip; y*=recip; z*=recip;
   return *this;
  }
  // nasobenie skalarom
  const tVec3 operator*(const float s) const {
   return tVec3(x*s,y*s,z*s);
  }
  // nasobenie skalarom
  friend const tVec3 operator*(const float s,const tVec3 &vec) {
   return vec*s;
  }
  // delenie skalarom
  const tVec3 operator/(const float s) const {
   const float recip=1.0f/s;
   return tVec3(recip*x,recip*y,recip*z);
  }
  // nasobenie vektorov
  const tVec3 operator*(const tVec3 &vec) const {
   return tVec3(x*vec.x,y*vec.y,z*vec.z);
  }
};

// 4-rozmerny vektor
struct tVec4 {
 float x,y,z,w;
};

// vrchol + texturovacia suradnica
struct tMatVT {
 unsigned int s; // vrchol bude/nebude vyuzity
 float x,y,z;    // suradnice vrcholu
 float u,v;      // texturovacie suradnice
};

// trieda premennych a funkcii (objektov) pre pracu s matematickymi vypoctami
class m3dmath {
public:
 m3dmath();          // konstruktor
 virtual ~m3dmath(); // destruktor

 // vypocet prvkov 3-rozmerneho vektora
 tVec3 Vector(tVec3 P1,tVec3 P2);
 // sucet vektorov
 tVec3 AddVector(tVec3 V1,tVec3 V2);
 // delenie vektora skalarom
 tVec3 DivVectorByScalar(tVec3 V,float S);
 // skalarny sucin dvoch vektorov
 float Dot(tVec3 V1,tVec3 V2);
 // vektorovy sucin dvoch vektorov
 tVec3 Cross(tVec3 V1,tVec3 V2);
 // normalizacia normaly
 tVec3 Normalize(tVec3 N);
private:
 tVec3 vector_result;
 float scalar_result;
 float magnitude;
};

#endif
