/*
  .3ds subory:

  Chunk - inf. blok .3ds suboru, v ktorom su zhromazdene udaje,
          identifikator inf. bloku urcuje, ake udaje sa v nom nachadzaju
  Sub-chunk - inf. blok moze obsahovat podblok, ktory moze obsahovat dalsi podblok,
              v inf. bloku moze byt rekurzivne umiestnene lubovolne mnozstvo podblokov
  Dlzka inf. bloku - inf. blok nema informaciu o dlzke svojich udajov, ale ma
                     informaciu o svojej celkovej dlzke
  Material - farba alebo textura priradena povrchu objektu, mateerial navyse uchovava
             informacie o odrazivosti, odlesku... povrchu objektu, tzn. o optickych
             vlastnostiach povrchu

  z 3dsinfo.txt:

  This is the representation of a chunk :

  Offset   Length    Name

  0        2         Chunk-ID
  2        4         Chunk-length = 6+n+m
  6        n         Data
  6+n      m         Sub-chunks

  0x4D4D          Main chunk
   0x3D3D         3D editor chunk
    0x4000        Object block (with name of your object)
     0x4100       Triangular mesh
       0x4110     Your vertices
       0x4120     Your faces
    0xAFFF        Material block
*/
// hlavickove subory *******************************************************
// GL a GLU kniznica
#include <GL/gl.h>
#include <GL/glu.h>
// funkcie SDL kniznice
#include <SDL/SDL.h>
#include <SDL/SDL_image.h>

// vlastny .h subor
#include "m3ds.h"

// konstruktor *************************************************************
m3ds::m3ds()
{

}

// destruktor **************************************************************
m3ds::~m3ds()
{
 // uvolnenie alokovanej pamate
 for (i=0;i<m3dsModel.ObjectsNum;i++) {
  delete [] m3dsModel.pObject[i].pVerts;
  delete [] m3dsModel.pObject[i].pFaces;
  delete [] m3dsModel.pObject[i].pTexVerts;
  delete [] m3dsModel.pObject[i].pNormals;
 }
}

// nacitanie inf. bloku ****************************************************
void m3ds::ReadChunk(tChunk *pChunk)
{
 pChunk->BRead=fread(&pChunk->ID,1,2,fp);      // Chunk-ID 2B

 pChunk->BRead+=fread(&pChunk->Length,1,4,fp); // Chunk-lenght 4B
}

// nacitanie znakov a ich nasledne ulozenie do retazca *********************
int m3ds::GetString(char *pBuf)
{
 i=0; fread(pBuf,1,1,fp); // 1B

 while (*(pBuf+i)!=0) {
  i++; fread((pBuf+i),1,1,fp); // 1B
 }

 return(strlen(pBuf)+1);
}

// import .3ds suboru s modelom ********************************************
int m3ds::imp3ds(char *fname)
{
 // otvorenie .3ds suboru - read binary
 fp=fopen(fname,"rb");
 if (!fp) {
  fprintf(stderr,"Unable to open the file: %s!",fname); return(1);
 }

 // vynulovanie identifikatorov textur
 for (j=0;j<MAX_TEXTURES;j++) TextureID[j]=0;
 // vynulovanie poctu objektov a materialov
 m3dsModel.ObjectsNum=0;
 m3dsModel.MaterialsNum=0;

 // priprava premennych pre nacitavanie jednotlivych inf. blokov .3ds suboru
 CurChunk=new tChunk;
 TmpChunk=new tChunk;

 // nacitanie 1. inf. bloku - musi byt MAIN, inac to nie je .3ds subor!
 ReadChunk(CurChunk);
 if (CurChunk->ID!=MAIN) {
  fprintf(stderr,"Unable to read MAIN chunk from file: %s!",fname);
  // zrusenie premennych pre nacitavanie jednotlivych inf. blokov .3ds suboru
  delete TmpChunk;
  delete CurChunk;
  return(1);
 }

 // rekurzivna funkcia pre nacitanie obsahu jednotlivych inf. blokov
 ProcessNextChunk(&m3dsModel,CurChunk);

 // zrusenie premennych pre nacitavanie jednotlivych inf. blokov .3ds suboru
 delete TmpChunk;
 delete CurChunk;

 // zatvorenie .3ds suboru
 fclose(fp);

 // vlastny vypocet normal, nakolko .3ds subor neobsahuje informacie o normalach
 //ComputeNormals(&m3dsModel);

 fprintf(stderr,"Textures:\n");

 // nacitanie textur
 SDL_Surface *TextureImage;

 for (i=0;i<m3dsModel.MaterialsNum;i++) {
  if (strlen(m3dsModel.pMaterial[i].MappingFile)>0) { // ak material ma texturu
   strcpy(path,"data/textures/");
   strcat(path,m3dsModel.pMaterial[i].MappingFile);

   TextureImage=IMG_Load(path);
   if (TextureImage==NULL) {
    fprintf(stderr,"\nError while IMG_Load: %s\n",path);
    //return(1);
    TextureImage=IMG_Load("data/textures/empty.tga"); // nahratie standardneho obrazku
   }

   glGenTextures(1,&TextureID[i]);
   glBindTexture(GL_TEXTURE_2D,TextureID[i]);
   glPixelStorei(GL_UNPACK_ALIGNMENT,1);
   glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
   glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
   glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
   glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
   gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB,TextureImage->w,TextureImage->h,
                     GL_BGR,GL_UNSIGNED_BYTE,TextureImage->pixels);

   // INFO: subor s texturou
   fprintf(stderr,"%i %s\n",TextureID[i],path);

   if (TextureImage) SDL_FreeSurface(TextureImage);
   TextureImage=NULL;
  }
  // ulozenie cisla textury
  m3dsModel.pMaterial[i].texID=i;
 }

 return(0);
}

// vykreslenie .3ds modelu *************************************************
int m3ds::render3ds(void)
{
 // pre vsetky objekty
 for (i=0;i<m3dsModel.ObjectsNum;i++) {
  if (m3dsModel.pObject.size()<=0) return(1);

  TmpObject=&m3dsModel.pObject[i];

  glColor4f(1.0f,1.0f,1.0f,1.0f);
  if (TmpObject->isTex) { // objekt ma texturu
   glEnable(GL_TEXTURE_2D);
   glBindTexture(GL_TEXTURE_2D,TextureID[TmpObject->texID]);
  } else { // objekt nema texturu - iba farbu
     glDisable(GL_TEXTURE_2D);
    }

  glBegin(GL_TRIANGLES);

   // pre vsetky steny
   for (j=0;j<TmpObject->FacesNum;j++) {
    // pre vsetky vrcholy
    for (k=0;k<3;k++) {
     l=TmpObject->pFaces[j].vIndex[k];
     // normala
     glNormal3f(TmpObject->pNormals[l].x,TmpObject->pNormals[l].y,TmpObject->pNormals[l].z);
     if (TmpObject->isTex) {
      if (TmpObject->pTexVerts) {
       // texturovacie suradnice
       glTexCoord2f(TmpObject->pTexVerts[l].x,TmpObject->pTexVerts[l].y);
      }
     } else {
        if (m3dsModel.pMaterial.size()&&TmpObject->texID>=0) {
         // farba
         glColor4f(m3dsModel.pMaterial[TmpObject->texID].DiffuseColor[0],
                   m3dsModel.pMaterial[TmpObject->texID].DiffuseColor[1],
                   m3dsModel.pMaterial[TmpObject->texID].DiffuseColor[2],1.0f);
        }
       }
     // suradnice vrcholu
     glVertex3f(TmpObject->pVerts[l].x,TmpObject->pVerts[l].y,TmpObject->pVerts[l].z);
    }
   }
  glEnd();
 }

 // nastavenie standardnej farby - biela
 glColor4f(1.0f,1.0f,1.0f,1.0f);

 return(0);
}

// vykreslenie normal .3ds modelu ******************************************
int m3ds::render3dsNormals(void)
{
 // pre vsetky objekty
 for (i=0;i<m3dsModel.ObjectsNum;i++) {
  if (m3dsModel.pObject.size()<=0) return(1);

  TmpObject=&m3dsModel.pObject[i];

  // ulozenie atributov osvetlenia, testu hlbky a texturovania
  glPushAttrib(GL_LIGHTING_BIT|GL_DEPTH_BUFFER_BIT|GL_TEXTURE_BIT);
  glDisable(GL_LIGHTING);   // vypnutie osvetlenia
  glDisable(GL_TEXTURE_2D); // vypnutie texturovania

  // farba - cervena
  glColor4f(1.0f,0.0f,0.0f,1.0f);

  glBegin(GL_LINES);

   // pre vsetky steny
   for (j=0;j<TmpObject->FacesNum;j++) {
    // pre vsetky vrcholy
    for (k=0;k<3;k++) {
     l=TmpObject->pFaces[j].vIndex[k];
     // suradnice vrcholu
     glVertex3f(TmpObject->pVerts[l].x,TmpObject->pVerts[l].y,TmpObject->pVerts[l].z);
     // suradnice vrcholu+normala
     glVertex3f(TmpObject->pVerts[l].x+TmpObject->pNormals[l].x/10.0f,
                TmpObject->pVerts[l].y+TmpObject->pNormals[l].y/10.0f,
                TmpObject->pVerts[l].z+TmpObject->pNormals[l].z/10.0f);
    }
   }

  glEnd();

  glPopAttrib();
 }

 // nastavenie standardnej farby - biela
 glColor4f(1.0f,1.0f,1.0f,1.0f);

 return(0);
}

// rekurzivna funkcia pre nacitanie obsahu jednotlivych inf. blokov ********
void m3ds::ProcessNextChunk(tModel *pModel,tChunk *pPrevChunk)
{
 tObject nObject;
 tMaterial nMaterial;
 unsigned short int version=0;
 int buf[50000]={0};

 // priprava premennej pre nacitavanie jednotlivych inf. blokov .3ds suboru
 CurChunk=new tChunk;

 // citanie obsahu inf. bloku a jeho podblokov az pokial sa nenacita tolko
 // Byte, kolko je dlzka celeho inf. bloku
 while (pPrevChunk->BRead<pPrevChunk->Length) {
  // nacitanie inf. bloku
  ReadChunk(CurChunk);

  switch (CurChunk->ID) {
   case VERSION: // nacitanie verzie .3ds suboru ---------------------------
    CurChunk->BRead+=fread(&version,1,CurChunk->Length-CurChunk->BRead,fp);
    // ak je verzia viac ako 3.0, moze to sposobovat urcite problemy there could be a problem
    if (version>0x03) fprintf(stderr,"This 3DS file is over version 3 so it may load incorrectly!");
   break;
   case EDITOR: // nacitanie inf. o editore - tento blok ma podbloky OBJECT a MATERIAL
    // nacitanie inf. bloku
    ReadChunk(TmpChunk);
    TmpChunk->BRead+=fread(&buf,1,TmpChunk->Length-TmpChunk->BRead,fp);
    CurChunk->BRead+=TmpChunk->BRead;

    // rekurzivne hladanie podblokov OBJECT a MATERIAL
    ProcessNextChunk(pModel, CurChunk);
   break;
   case OBJECT: // nacitanie inf. o vrcholoch, stenach... ------------------
    // zvacsenie poctu objektov
    pModel->ObjectsNum++;
    // pridanie noveho objektu do pola
    pModel->pObject.push_back(nObject);
    // inicializacia obsahu objektu
    memset(&(pModel->pObject[pModel->ObjectsNum-1]),0,sizeof(tObject));
    // nacitanie nazvu objektu
    CurChunk->BRead+=GetString(pModel->pObject[pModel->ObjectsNum-1].Name);
    fprintf(stderr,"Object name: %s\n",pModel->pObject[pModel->ObjectsNum-1].Name);

    // rekurzivne hladanie podbloku s inf. o objekte
    ProcessNextObjectChunk(pModel,&(pModel->pObject[pModel->ObjectsNum-1]),CurChunk);
   break;
   case MATERIAL: // nacitanie inf. o materialy ----------------------------
    // zvacsenie poctu materialov
    pModel->MaterialsNum++;
    // pridanie noveho materialu do pola
    pModel->pMaterial.push_back(nMaterial);
    // inicializacia obsahu materialu
    memset(&(pModel->pMaterial[pModel->MaterialsNum-1]),0,sizeof(tMaterial));

    // rekurzivne hladanie podbloku s inf. o materialy
    ProcessNextMaterialChunk(pModel,CurChunk);
   break;
   default: // -------------------------------------------------------------
    // preskocenie dalsich inf. blokov .3ds suboru
    CurChunk->BRead+=fread(buf,1,CurChunk->Length-CurChunk->BRead,fp);
   break;
  }

  pPrevChunk->BRead+=CurChunk->BRead;
 }

 // zrusenie premennej pre nacitavanie jednotlivych inf. blokov .3ds suboru
 delete CurChunk;
 CurChunk=pPrevChunk;
}

// rekurzivna funkcia pre nacitanie obsahu inf. bloku OBJECT ***************
void m3ds::ProcessNextObjectChunk(tModel *pModel,tObject *pObject,tChunk *pPrevChunk)
{
 char TmpName[255]={0};
 unsigned short int index=0;
 int buf[50000]={0};

 // priprava premennej pre nacitavanie jednotlivych inf. blokov .3ds suboru
 CurChunk=new tChunk;

 // citanie obsahu inf. bloku a jeho podblokov az pokial sa nenacita tolko
 // Byte, kolko je dlzka celeho inf. bloku
 while (pPrevChunk->BRead<pPrevChunk->Length) {
  // nacitanie inf. bloku
  ReadChunk(CurChunk);

  switch (CurChunk->ID) {
   case TRI_MESH: // inforamcie o sietovom modely objektu ------------------
    // rekurzivne hladanie podbloku s inf. o sietovom modely objektu
    ProcessNextObjectChunk(pModel,pObject,CurChunk);
   break;
   case VERT_LIST: // ------------------------------------------------------
    // nacitanie poctu vrcholov
    CurChunk->BRead+=fread(&(pObject->VertsNum),1,2,fp);
    // INFO: pocet vrcholov objektu
    fprintf(stderr,"Number of vertices: %d\n",pObject->VertsNum);
    // priprava pola vrcholov
    pObject->pVerts=new tVec3 [pObject->VertsNum];
    memset(pObject->pVerts,0,pObject->VertsNum*sizeof(tVec3));

    // nacitanie suradnic vrcholov
    CurChunk->BRead+=fread(pObject->pVerts,1,CurChunk->Length-CurChunk->BRead,fp);
   break;
   case FACES_DESCR: // ----------------------------------------------------
    // nacitanie poctu stien
    CurChunk->BRead+=fread(&pObject->FacesNum,1,2,fp);
    // INFO: pocet vrcholov objektu
    fprintf(stderr,"Number of faces: %d\n",pObject->FacesNum);
    // priprava pola stien
    pObject->pFaces=new tFace [pObject->FacesNum];
    memset(pObject->pFaces,0,pObject->FacesNum*sizeof(tFace));

    // nacitanie indexov do pola stien
    for (i=0;i<pObject->FacesNum;i++) {
     for (j=0;j<4;j++) {
      CurChunk->BRead+=fread(&index,1,sizeof(index),fp);
      if (j<3) { // pozn.: 4. hodnota je tzv. indikator viditelnosti (3D Studio Max)
       pObject->pFaces[i].vIndex[j]=index;
      }
     }
    }
   break;
   case FACES_MATLIST: // --------------------------------------------------
    // nacitanie nazvu materialu
    CurChunk->BRead += GetString(TmpName);

    // otestovanie, ci sa uz rovnaky material nachadza v poli
    for (i=0;i<pModel->MaterialsNum;i++) {
     if (strcmp(TmpName,pModel->pMaterial[i].Name)==0) {
      // material sa uz v poli nachadza, objektu sa prideli patricne cislo textury
      pObject->texID=i;
      if (strlen(pModel->pMaterial[i].MappingFile)>0) {
       // objekt ma texturu
       pObject->isTex=true;
      }
      break;
     }
     else { // material sa v poli nenachadza
      pObject->texID=-1;
     }
    }
    // preskocenie dalsich inf. v aktualnom bloku
    CurChunk->BRead+=fread(buf,1,CurChunk->Length-CurChunk->BRead,fp);
   break;
   case MAPCOORDS: // ------------------------------------------------------
    // pocet texturovacich suradnic
    CurChunk->BRead+=fread(&pObject->TexVertsNum,1,2,fp);
    // INFO: pocet texturovacich suradnic
    fprintf(stderr,"Number of vertices (for UV): %d\n",pObject->TexVertsNum);
    // priprava pola teturovacich suradnic
    pObject->pTexVerts=new tVec2 [pObject->TexVertsNum];
    memset(pObject->pTexVerts,0,pObject->TexVertsNum*sizeof(tVec2));

    // nacitanie texturovacich suradnic
    CurChunk->BRead+=fread(pObject->pTexVerts,1,CurChunk->Length-CurChunk->BRead,fp);

    // INFO: vypis nacitanych texturovacich suradnic
    //for(int xx=0;xx<pObject->TexVertsNum;xx++) {
    // fprintf(stderr,"%f,%f\n",pObject->pTexVerts[xx].x,pObject->pTexVerts[xx].y);
    //}
   break;
   default: // -------------------------------------------------------------
    // preskocenie dalsich inf. blokov .3ds suboru
    CurChunk->BRead+=fread(buf,1,CurChunk->Length-CurChunk->BRead,fp);
   break;
  }

  pPrevChunk->BRead+=CurChunk->BRead;
 }

 // zrusenie premennej pre nacitavanie jednotlivych inf. blokov .3ds suboru
 delete CurChunk;
 CurChunk=pPrevChunk;
}

// rekurzivna funkcia pre nacitanie obsahu inf. bloku MATERIAL *************
void m3ds::ProcessNextMaterialChunk(tModel *pModel,tChunk *pPrevChunk)
{
 int buf[50000]={0};
 unsigned char DiffuseC[3]={0,0,0};

 // priprava premennej pre nacitavanie jednotlivych inf. blokov .3ds suboru
 CurChunk=new tChunk;

 while (pPrevChunk->BRead<pPrevChunk->Length) {
  // nacitanie inf. bloku
  ReadChunk(CurChunk);

  switch (CurChunk->ID) {
   case MATNAME: // nacitanie nazvu materialu ------------------------------
    CurChunk->BRead+=fread(pModel->pMaterial[pModel->MaterialsNum-1].Name,1,CurChunk->Length-CurChunk->BRead,fp);
    // INFO: nazov materialu
    fprintf(stderr,"Material name: %s\n",pModel->pMaterial[pModel->MaterialsNum-1].Name);
   break;
   case MATDIFFUSE: // nacitanie difuznej zlozky farby materialu -----------
    // nacitanie inf. bloku
    ReadChunk(TmpChunk);
    // nacitanie R,G,B zloziek farby
    TmpChunk->BRead+=fread(DiffuseC,1,TmpChunk->Length-TmpChunk->BRead,fp);

    pModel->pMaterial[pModel->MaterialsNum-1].DiffuseColor[0]=(float) DiffuseC[0]/255.0f;
    pModel->pMaterial[pModel->MaterialsNum-1].DiffuseColor[1]=(float) DiffuseC[1]/255.0f;
    pModel->pMaterial[pModel->MaterialsNum-1].DiffuseColor[2]=(float) DiffuseC[2]/255.0f;

    // INFO: difuzna zlozka farby
    //fprintf(stderr,"diffuse color: %f %f %f\n",pModel->pMaterial[pModel->MaterialsNum-1].DiffuseColor[0],
    //                                           pModel->pMaterial[pModel->MaterialsNum-1].DiffuseColor[1],
    //                                           pModel->pMaterial[pModel->MaterialsNum-1].DiffuseColor[2]);
    // Add the bytes read to our chunk
    CurChunk->BRead+=TmpChunk->BRead;
   break;
   case MATMAP: // ---------------------------------------------------------
    // rekurzivne hladanie podbloku s inf. o texture
    ProcessNextMaterialChunk(pModel,CurChunk);
   break;
   case MATMAPFILE: // nacitanie suboru s texturou -------------------------
    CurChunk->BRead+=fread(pModel->pMaterial[pModel->MaterialsNum-1].MappingFile,
                           1,CurChunk->Length-CurChunk->BRead,fp);
    // INFO: nazov suboru s texturou
    fprintf(stderr,"Mapping filename: %s\n",pModel->pMaterial[pModel->MaterialsNum-1].MappingFile);
   break;
   case V_SCALE: // --------------------------------------------------------
    CurChunk->BRead+=fread(&(pModel->pMaterial[pModel->MaterialsNum-1].Vscale),1,sizeof(float),fp);
   break;
   case U_SCALE: // --------------------------------------------------------
    CurChunk->BRead+=fread(&(pModel->pMaterial[pModel->MaterialsNum-1].Uscale),1,sizeof(float),fp);
   break;
   default: // -------------------------------------------------------------
    // preskocenie dalsich inf. blokov .3ds suboru
    CurChunk->BRead+=fread(buf,1,CurChunk->Length-CurChunk->BRead,fp);
   break;
  }

  pPrevChunk->BRead+=CurChunk->BRead;
 }

 // zrusenie premennej pre nacitavanie jednotlivych inf. blokov .3ds suboru
 delete CurChunk;
 CurChunk=pPrevChunk;
}

// vypocet normal **********************************************************
int m3ds::ComputeNormals(tModel *pModel)
{
 tVec3 vVec1,vVec2,vNormal,vPoly[3];

 // vypocet iba ak su v scene nejake objekty
 if (pModel->ObjectsNum==0) return(1);

 // pre vsetky objekty
 for (k=0;k<pModel->ObjectsNum;k++) {
  TmpObject=&(pModel->pObject[k]);

  // priprava docasnych poli
  tVec3 *pNormals=new tVec3 [TmpObject->FacesNum];
  tVec3 *pTmpNormals=new tVec3 [TmpObject->FacesNum];
  // priprava pola normal
  TmpObject->pNormals=new tVec3 [TmpObject->VertsNum];
  memset(TmpObject->pNormals,0,TmpObject->VertsNum*sizeof(tVec3));

  // vypocet normal vsetkych stien
  for (i=0;i<TmpObject->FacesNum;i++) {
   // ziskanie suradnic vrcholov
   vPoly[0]=TmpObject->pVerts[TmpObject->pFaces[i].vIndex[0]];
   vPoly[1]=TmpObject->pVerts[TmpObject->pFaces[i].vIndex[1]];
   vPoly[2]=TmpObject->pVerts[TmpObject->pFaces[i].vIndex[2]];
   // vypocet dvoch vektorov
   vVec1=m_3dmath.Vector(vPoly[1],vPoly[0]);
   vVec2=m_3dmath.Vector(vPoly[2],vPoly[1]);
   // vektorovy sucin dvoch vektorov = normala
   vNormal=m_3dmath.Cross(vVec1,vVec2);
   // docasne ulozenie normaly
   pTmpNormals[i]=vNormal;
   // normalizacia normaly
   vNormal=m_3dmath.Normalize(vNormal);
   // docasne ulozenie normalizovanej normaly
   pNormals[i]=vNormal;
   // INFO: normaly stien
   //fprintf(stderr,"%f, %f, %f\n",pNormals[i].x,pNormals[i].y,pNormals[i].z);
  }

  tVec3 vZero(0.0,0.0,0.0); // "nulovy" vektor
  tVec3 vSum=vZero;         // pre sucet dvoch normal
  int shared=0;             // pocet totoznych vrcholov

  // vypocet normal vsetkych vrcholov
  for (i=0;i<TmpObject->VertsNum;i++) { // pre vsetky vrcholy
   for (j=0;j<TmpObject->FacesNum;j++) { // pre vsetky steny
    if (TmpObject->pFaces[j].vIndex[0]==i||
        TmpObject->pFaces[j].vIndex[1]==i||
        TmpObject->pFaces[j].vIndex[2]==i) {
     // sucet dvoch normal - povodnej a novej normaly steny, ak je aspon jeden
     // jej vrchol totozny z porovnavanym
     vSum=m_3dmath.AddVector(vSum,pTmpNormals[j]);
     shared++;
    }
   }
   // vypocet vyslednej normaly vrcholu - potrebne delit poctom totoznych vrcholov
   TmpObject->pNormals[i]=m_3dmath.DivVectorByScalar(vSum,float(shared));
   TmpObject->pNormals[i]=m_3dmath.Normalize(TmpObject->pNormals[i]);

   // INFO: pocet totoznych vrcholov
   //fprintf(stderr,"%d\n",shared);

   vSum=vZero;
   shared=0;
  }

  // zrusenie docasnych poli
  delete [] pTmpNormals;
  delete [] pNormals;
 }

 return(0);
}
