#ifndef _M3DS_H_
#define _M3DS_H_

#define MAX_TEXTURES 100 // max. pocet textur

#include <vector>
using namespace std;

// informacne bloky .3ds suboru
#define MAIN          0x4D4D // uvodny inf. blok .3ds suboru

// MAIN
#define VERSION       0x0002 // verzia .3ds suboru
#define EDITOR        0x3D3D // inf. o 3D editore
#define OBJECT        0x4000 // inf. o vrcholoch, stenach...
#define MATERIAL      0xAFFF // inf. o materialy

// OBJECT
#define TRI_MESH      0x4100 // sietovy model objektu

// TRI_MESH
#define VERT_LIST     0x4110 // vrcholy sietoveho modelu
#define FACES_DESCR   0x4120 // steny sietoveho modelu
#define FACES_MATLIST 0x4130 // material stien
#define MAPCOORDS     0x4140 // texturovacie suradnice

// MATERIAL
#define MATNAME       0xA000 // nazov materialu
#define MATDIFFUSE    0xA020 // difuzna zlozka farby
#define MATMAP        0xA200 // nazov textury
#define MATMAPFILE    0xA300 // subor s texturou
#define V_SCALE       0xA354 // posun textury v smere V
#define U_SCALE       0xA356 // posun textury v smere U

#include "m3dmath.h"

// stena trojuholnika - indexy do poli vrcholov a texturovacich suradnic
struct tFace {
 unsigned int vIndex[3];
 unsigned int tcIndex[3];
};

// indexy do pola vrcholov (3 vrcholy, indikator viditelnosti)
struct tIndices {
 unsigned short int a,b,c,bVisible;
};

// informacny blok .3ds suboru
struct tChunk {
 unsigned short int ID; // Chunk-ID 2B
 unsigned int Length;   // Chunk-lenght 4B
 unsigned int BRead;    // pocet nacitanych Byte
};

// objekt
struct tObject {
 char Name[255]; // nazov objektu

 unsigned int VertsNum;    // pocet vrcholov
 tVec3 *pVerts;            // pole vrcholov

 unsigned int FacesNum;    // pocet stien
 tFace *pFaces;            // pole stien

 unsigned int TexVertsNum; // pocet texturovacich suradnic
 tVec2 *pTexVerts;         // pole texturovacich suradnic

 tVec3 *pNormals;          // pole normal

 bool isTex;         // objekt ma/nema texturu
 unsigned int texID; // cislo textury
};

// material priradeny objektu
struct tMaterial {
 char Name[255];        // nazov materialu
 float DiffuseColor[3]; // difuzna zlozka farby (R,G,B)
 char MappingFile[255]; // nazov textury
 unsigned int texID;    // identifikator textury
 float Vscale;          // opakovanie sa textury v smere V
 float Uscale;          // opakovanie sa textury v smere U
 float Uoffst;          // posun textury v smere U
 float VOffst;          // posun textury v smere V
};

// .3ds model
struct tModel {
 unsigned int ObjectsNum; // pocet objektov
 vector<tObject> pObject; // pole objektov

 unsigned int MaterialsNum;   // pocet materialov objektov
 vector<tMaterial> pMaterial; // pole materialov
};

// trieda premennych a funkcii (objektov) pre pracu s .3ds subormi
class m3ds {
public:
 // konstruktor
 m3ds();
 // destruktor
 virtual ~m3ds();

 tModel m3dsModel; // .3ds model

 tObject *TmpObject; // pomocna premenna

 unsigned int TextureID[MAX_TEXTURES]; // identifikatory textur

 int imp3ds(char *fname); // import .3ds suboru s modelom

 int render3ds(void);        // vykreslenie .3ds modelu
 int render3dsNormals(void); // vykreslenie normal .3ds modelu
private:
 unsigned int i,j,k,l; // pomocne premenne

 char path[256]; // cesta k suboru

 FILE *fp; // subor

 // premenne urcene na nacitavanie jednotlivych inf. blokov .3ds suboru
 tChunk *CurChunk;
 tChunk *TmpChunk;

 // nacitanie inf. bloku
 void ReadChunk(tChunk *pChunk);
 // nacitanie znakov a ich nasledne ulozenie do retazca
 int GetString(char *pBuf);

 // rekurzivna funkcia pre nacitanie obsahu jednotlivych inf. blokov
 void ProcessNextChunk(tModel *pModel,tChunk *pPrevChunk);
 // rekurzivna funkcia pre nacitanie obsahu inf. bloku OBJECT
 void ProcessNextObjectChunk(tModel *pModel,tObject *pObject,tChunk *pPrevChunk);
 // rekurzivna funkcia pre nacitanie obsahu inf. bloku MATERIAL
 void ProcessNextMaterialChunk(tModel *pModel,tChunk *pPrevChunk);

 // vypocet normal
 m3dmath m_3dmath;
 int ComputeNormals(tModel *pModel);
};

#endif
