// hlavickove subory *******************************************************
// systemove funkcie
#include <math.h>
// GL kniznica
#include <GL/gl.h>

// vlastny .h subor
#include "mFrustum.h"

// zoznam stran (rovin) Frustum
enum FrustumSide
{
 RIGHT =0, // prava strana (rovina) Frustum
 LEFT  =1, // lava strana (rovina) Frustum
 BOTTOM=2, // spodna strana (rovina) Frustum
 TOP   =3, // vrchna strana (rovina) Frustum
 BACK  =4, // zadna strana (rovina) Frustum
 FRONT =5  // predna strana (rovina) Frustum
};

// zoznam prvkov rivnic stran (rovin) Frustum
enum PlaneData
{
 A=0, // normala X
 B=1, // normala Y
 C=2, // normala Z
 D=3  // vzdialenost strany (roviny) od pociatku suradnicovej sustavy
};

// konstruktor *************************************************************
mFrustum::mFrustum()
{

}

// destruktor **************************************************************
mFrustum::~mFrustum()
{

}

// normalizacia prvkov rovnice strany (roviny) Frustum *********************
void mFrustum::NormalizePlane(GLfloat F[6][4],GLuint S)
{
 // vypocet magnitudy a reciprocnej hodnoty magnitudy normaly strany (roviny)
 // magn=sqrt(x^2+y^2+z^2)
 // recip_magn=1/magn
 magn=(GLfloat) sqrt(F[S][A]*F[S][A]+
                     F[S][B]*F[S][B]+
                     F[S][C]*F[S][C]);
 if (magn==0.0f) magn=1.0f;
 recip_magn=1.0f/magn;

 // nasobenie prvkov rovnice strany (roviny)
 // reciprocnou hodnotou magnitudy jej normaly
 F[S][A]*=recip_magn;
 F[S][B]*=recip_magn;
 F[S][C]*=recip_magn;
 F[S][D]*=recip_magn;
}

// vypocet rovnic stran (rovin) Frustum ************************************
void mFrustum::FrustumUpdate(void)
{
 // ziskanie projekcnej matice
 glGetFloatv(GL_PROJECTION_MATRIX,proj);

 // nacitanie modelview matice
 glGetFloatv(GL_MODELVIEW_MATRIX,modv);

 // vypocet orezavacich stran (rovin) kombinaciou matic
 // ziskavaju sa prvky rovnic stran (rovin):
 // A,B,C - normala X,Y,Z
 // D - vzialenost strany (roviny) od pociatku suradnicovej sustavy
 clip[ 0]=modv[ 0]*proj[ 0]+modv[ 1]*proj[ 4]+modv[ 2]*proj[ 8]+modv[ 3]*proj[12];
 clip[ 1]=modv[ 0]*proj[ 1]+modv[ 1]*proj[ 5]+modv[ 2]*proj[ 9]+modv[ 3]*proj[13];
 clip[ 2]=modv[ 0]*proj[ 2]+modv[ 1]*proj[ 6]+modv[ 2]*proj[10]+modv[ 3]*proj[14];
 clip[ 3]=modv[ 0]*proj[ 3]+modv[ 1]*proj[ 7]+modv[ 2]*proj[11]+modv[ 3]*proj[15];

 clip[ 4]=modv[ 4]*proj[ 0]+modv[ 5]*proj[ 4]+modv[ 6]*proj[ 8]+modv[ 7]*proj[12];
 clip[ 5]=modv[ 4]*proj[ 1]+modv[ 5]*proj[ 5]+modv[ 6]*proj[ 9]+modv[ 7]*proj[13];
 clip[ 6]=modv[ 4]*proj[ 2]+modv[ 5]*proj[ 6]+modv[ 6]*proj[10]+modv[ 7]*proj[14];
 clip[ 7]=modv[ 4]*proj[ 3]+modv[ 5]*proj[ 7]+modv[ 6]*proj[11]+modv[ 7]*proj[15];

 clip[ 8]=modv[ 8]*proj[ 0]+modv[ 9]*proj[ 4]+modv[10]*proj[ 8]+modv[11]*proj[12];
 clip[ 9]=modv[ 8]*proj[ 1]+modv[ 9]*proj[ 5]+modv[10]*proj[ 9]+modv[11]*proj[13];
 clip[10]=modv[ 8]*proj[ 2]+modv[ 9]*proj[ 6]+modv[10]*proj[10]+modv[11]*proj[14];
 clip[11]=modv[ 8]*proj[ 3]+modv[ 9]*proj[ 7]+modv[10]*proj[11]+modv[11]*proj[15];

 clip[12]=modv[12]*proj[ 0]+modv[13]*proj[ 4]+modv[14]*proj[ 8]+modv[15]*proj[12];
 clip[13]=modv[12]*proj[ 1]+modv[13]*proj[ 5]+modv[14]*proj[ 9]+modv[15]*proj[13];
 clip[14]=modv[12]*proj[ 2]+modv[13]*proj[ 6]+modv[14]*proj[10]+modv[15]*proj[14];
 clip[15]=modv[12]*proj[ 3]+modv[13]*proj[ 7]+modv[14]*proj[11]+modv[15]*proj[15];

 // vypocet prvkov rovnice strany (roviny) Frustum
 plane[RIGHT][A]=clip[ 3]-clip[ 0];
 plane[RIGHT][B]=clip[ 7]-clip[ 4];
 plane[RIGHT][C]=clip[11]-clip[ 8];
 plane[RIGHT][D]=clip[15]-clip[12];
 // normalizacia prvkov rovnice strany (roviny) Frustum
 NormalizePlane(plane,RIGHT);

 // vypocet prvkov rovnice strany (roviny) Frustum
 plane[LEFT][A]=clip[ 3]+clip[ 0];
 plane[LEFT][B]=clip[ 7]+clip[ 4];
 plane[LEFT][C]=clip[11]+clip[ 8];
 plane[LEFT][D]=clip[15]+clip[12];
 // normalizacia prvkov rovnice strany (roviny) Frustum
 NormalizePlane(plane,LEFT);

 // vypocet prvkov rovnice strany (roviny) Frustum
 plane[BOTTOM][A]=clip[ 3]+clip[ 1];
 plane[BOTTOM][B]=clip[ 7]+clip[ 5];
 plane[BOTTOM][C]=clip[11]+clip[ 9];
 plane[BOTTOM][D]=clip[15]+clip[13];
 // normalizacia prvkov rovnice strany (roviny) Frustum
 NormalizePlane(plane,BOTTOM);

 // vypocet prvkov rovnice strany (roviny) Frustum
 plane[TOP][A]=clip[ 3]-clip[ 1];
 plane[TOP][B]=clip[ 7]-clip[ 5];
 plane[TOP][C]=clip[11]-clip[ 9];
 plane[TOP][D]=clip[15]-clip[13];
 // normalizacia prvkov rovnice strany (roviny) Frustum
 NormalizePlane(plane,TOP);

 // vypocet prvkov rovnice strany (roviny) Frustum
 plane[BACK][A]=clip[ 3]-clip[ 2];
 plane[BACK][B]=clip[ 7]-clip[ 6];
 plane[BACK][C]=clip[11]-clip[10];
 plane[BACK][D]=clip[15]-clip[14];
 // normalizacia prvkov rovnice strany (roviny) Frustum
 NormalizePlane(plane,BACK);

 // vypocet prvkov rovnice strany (roviny) Frustum
 plane[FRONT][A]=clip[ 3]+clip[ 2];
 plane[FRONT][B]=clip[ 7]+clip[ 6];
 plane[FRONT][C]=clip[11]+clip[10];
 plane[FRONT][D]=clip[15]+clip[14];
 // normalizacia prvkov rovnice strany (roviny) Frustum
 NormalizePlane(plane,FRONT);
}

/*
  pre test, ci je bod (resp. trojuholnik, gula, kocka...) vnutri Frustum
  pouzivame predpoklad, ze vsetky normaly stran (rovin) Frustum smeruju dovnutra
  Frustum priestoru a teda pokial je bod (resp. jeden, alebo vsetky vrcholy
  trojuholnika, gule, kocky...) pred vsetkymi 6-timi stranami Frustum, je vnutri Frustum
*/

// test, ci je bod vo Frustum **********************************************
bool mFrustum::PointInFrustum(GLfloat X,GLfloat Y,GLfloat Z)
{
 // test so vsetkymi 6-timi stranami (rovinami) Frustum
 for (i=0;i<6;i++) {
  if (plane[i][A]*X+plane[i][B]*Y+plane[i][C]*Z+plane[i][D]<=0) {
   // bod je za stranou (rovinou) => je vonku z Frustum
   return false;
  }
 }
 // bod je pred vsetkymi 6-timi stranami (rovinami) => je vo Frustum
 return true;
}

// test, ci je trojuholnik vo Frustum **************************************
bool mFrustum::TriangleInFrustum(GLfloat X1,GLfloat Y1,GLfloat Z1,
                                 GLfloat X2,GLfloat Y2,GLfloat Z2,
                                 GLfloat X3,GLfloat Y3,GLfloat Z3)
{
 // testuju sa vsetky 3 vrcholy trojuholnika, trojuholnik bude vonku z Frustum
 // iba v pripade, ze vsetky jeho 3 vrcholy lezia vonku z Frustum,
 // v opacnom pripade bude trojuholnik vo Frustum

 if (PointInFrustum(X1,Y1,Z1)) return true;
 if (PointInFrustum(X2,Y2,Z2)) return true;
 if (PointInFrustum(X3,Y3,Z3)) return true;

 // vsetky 3 vrcholy lezia vonku z Frustum => trojuholnik je vonku z Frustum
 return false;
}

// test, ci je gula vo Frustum *********************************************
bool mFrustum::SphereInFrustum(GLfloat X,GLfloat Y,GLfloat Z,GLfloat R)
{
 // test so vsetkymi 6-timi stranami (rovinami) Frustum
 for (i=0;i<6;i++) {
  if (plane[i][A]*X+plane[i][B]*Y+plane[i][C]*Z+plane[i][D]<=R) {
   // gula je za stranou (rovinou) => je vonku z Frustum
   return false;
  }
 }
 // gula je pred vsetkymi 6-timi stranami (rovinami) => je vo Frustum
 return true;
}

// test, ci je kocka vo Frustum ********************************************
bool mFrustum::CubeInFrustum(GLfloat X,GLfloat Y,GLfloat Z,GLfloat S)
{
 // testuje sa vsetkych 8 vrcholov kocky, kocka bude vonku z Frustum
 // iba v pripade, ze vsetkych jej 8 vrcholov lezi za niektorou zo
 // stran (rovin) Frustum, v opacnom pripade bude kocka vo Frustum

 // test so vsetkymi 6-timi stranami (rovinami) Frustum
 for (i=0;i<6;i++) {
  if (plane[i][A]*(X-S)+plane[i][B]*(Y-S)+plane[i][C]*(Z-S)+plane[i][D]>0)
   continue; // ak je vrchol pred stranou (rovinou) testuje sa dalsia strana (rovina)

  // v opacnom pripade sa testuju dalsie vrcholy:
  if (plane[i][A]*(X+S)+plane[i][B]*(Y-S)+plane[i][C]*(Z-S)+plane[i][D]>0)
   continue;
  if (plane[i][A]*(X-S)+plane[i][B]*(Y+S)+plane[i][C]*(Z-S)+plane[i][D]>0)
   continue;
  if (plane[i][A]*(X+S)+plane[i][B]*(Y+S)+plane[i][C]*(Z-S)+plane[i][D]>0)
   continue;
  if (plane[i][A]*(X-S)+plane[i][B]*(Y-S)+plane[i][C]*(Z+S)+plane[i][D]>0)
   continue;
  if (plane[i][A]*(X+S)+plane[i][B]*(Y-S)+plane[i][C]*(Z+S)+plane[i][D]>0)
   continue;
  if (plane[i][A]*(X-S)+plane[i][B]*(Y+S)+plane[i][C]*(Z+S)+plane[i][D]>0)
   continue;
  if (plane[i][A]*(X+S)+plane[i][B]*(Y+S)+plane[i][C]*(Z+S)+plane[i][D]>0)
   continue;
  // kocka je za stranou (rovinou) => je vonku z Frustum
  return false;
 }
 // kocka je pred vsetkymi 6-timi stranami (rovinami) => je vo Frustum
 return true;
}

// test, ci je kocka vo Frustum ********************************************
bool mFrustum::CubeInFrustum2(GLfloat X1,GLfloat X2,
                              GLfloat Y1,GLfloat Y2,
                              GLfloat Z1,GLfloat Z2)
{
 // testuje sa vsetkych 8 vrcholov kocky, kocka bude vonku z Frustum
 // iba v pripade, ze vsetkych jej 8 vrcholov lezi za niektorou zo
 // stran (rovin) Frustum, v opacnom pripade bude kocka vo Frustum

 // test so vsetkymi 6-timi stranami (rovinami) Frustum
 for (i=0;i<6;i++) {
  if (plane[i][A]*X1+plane[i][B]*Y1+plane[i][C]*Z1+plane[i][D]>0)
   continue; // ak je vrchol pred stranou (rovinou) testuje sa dalsia strana (rovina)

  // v opacnom pripade sa testuju dalsie vrcholy:
  if (plane[i][A]*X2+plane[i][B]*Y1+plane[i][C]*Z1+plane[i][D]>0)
   continue;
  if (plane[i][A]*X2+plane[i][B]*Y2+plane[i][C]*Z1+plane[i][D]>0)
   continue;
  if (plane[i][A]*X1+plane[i][B]*Y2+plane[i][C]*Z1+plane[i][D]>0)
   continue;
  if (plane[i][A]*X1+plane[i][B]*Y1+plane[i][C]*Z2+plane[i][D]>0)
   continue;
  if (plane[i][A]*X2+plane[i][B]*Y1+plane[i][C]*Z2+plane[i][D]>0)
   continue;
  if (plane[i][A]*X2+plane[i][B]*Y2+plane[i][C]*Z2+plane[i][D]>0)
   continue;
  if (plane[i][A]*X1+plane[i][B]*Y2+plane[i][C]*Z2+plane[i][D]>0)
   continue;
  // kocka je za stranou (rovinou) => je vonku z Frustum
  return false;
 }
 // kocka je pred vsetkymi 6-timi stranami (rovinami) => je vo Frustum
 return true;
}
