// hlavickove subory *******************************************************
// systemove funkcie
#include <stdio.h>
#include <stdlib.h>
// GL kniznica
#include <GL/gl.h>

// vlastny .h subor
#include "mOcttree.h"

// konstruktor *************************************************************
mOcttree::mOcttree()
{
 TREE_SIZE=2; // hlbka oktanoveho stromu, POZOR na spravne zrusenie vsetkych
              // hlbok oktanoveho stromu!

 oct_tree=NULL; // rodicovsky list oktanoveho stromu
 nodes_num=0;   // pocet listov oktanoveho stromu

 octtree_tri_num=0; // celkovy pocet trojuholnikov oktanoveho stromu

 tests_num=0; // pocet testov na viditelnost listov oktanoveho stromu

 // nie su prevzate ziadne trojuholniky z triedy mTriangle
 first_tri=NULL; tmp_tri=NULL;
 tri_num=0;
 act_tri=NULL;
 act_tri_num=0;

 // nie je prevzaty ziadny Frustum
 OGLFrustum=NULL;

 // pocitadla vyhradenej/uvolnenej pamate
 alloc_mem=0; free_mem=0;
}

// destruktor **************************************************************
mOcttree::~mOcttree()
{
 Octtree_destroy(); // zrusenie oktanoveho stromu
}

// vytvorenie oktanoveho stromu ********************************************
int mOcttree::Octtree_create(mTriangle *T,GLuint N)
{
 // trojuholniky z triedy mTriangle
 first_tri=T->first_tri; tmp_tri=first_tri;
 // celkovy pocet trojuholnikov z triedy mTriangle
 tri_num=N;

 // zacatie spocitavania velkosti pamate, ktoru obsadi oktanovy strom
 alloc_mem=0;

 oct_tree=new _tree_node; // vytvorenie rodicovskeho listu oktanoveho stromu
 if (oct_tree==NULL) {
  fprintf(stderr,"Chyba vyhradenia pamate pre rodicovsky uzol oktanoveho stromu!\n"); return(1);
 }
 alloc_mem+=sizeof(_tree_node);

 // ziskanie hranic rodicovskeho listu (kocky) oktanoveho stromu
 oct_tree->minx=99999.9f; oct_tree->maxx=-99999.9f;
 oct_tree->miny=99999.9f; oct_tree->maxy=-99999.9f;
 oct_tree->minz=99999.9f; oct_tree->maxz=-99999.9f;

 // pre vsetky trojuholniky z triedy mTriangle
 for (a=0;a<N;a++) {
  // najdenie "najmenseho" vrcholu vsetkych trojuholnikov
  if (tmp_tri->minx<oct_tree->minx) oct_tree->minx=tmp_tri->minx;
  if (tmp_tri->miny<oct_tree->miny) oct_tree->miny=tmp_tri->miny;
  if (tmp_tri->minz<oct_tree->minz) oct_tree->minz=tmp_tri->minz;

  // najdenie "najvacsieho" vrcholu vsetkych trojuholnikov
  if (tmp_tri->maxx>oct_tree->maxx) oct_tree->maxx=tmp_tri->maxx;
  if (tmp_tri->maxy>oct_tree->maxy) oct_tree->maxy=tmp_tri->maxy;
  if (tmp_tri->maxz>oct_tree->maxz) oct_tree->maxz=tmp_tri->maxz;

  // nasledujuci trojuholnik v zozname
  tmp_tri=tmp_tri->next;
 }
 // identifikator rodicovskeho listu
 oct_tree->id=nodes_num;
 nodes_num++;
 // vsetky podlisty budu zatial prazdne
 for (a=0;a<8;a++) oct_tree->node[a]=NULL;
 // rodicovsky list nema ziadne trojuholniky
 oct_tree->node_tri=NULL;
 oct_tree->node_tri_num=0;
 // nastavenie indikatora zobrazenia listu
 oct_tree->node_draw_bit=true;

 node_level=0;
 node_add(oct_tree); // rekurzivne!!! pridanie listov do oktanoveho stromu

 // INFO: vyhradena pamat
 //fprintf(stderr,"Velkost pamate vyhradenej pre oktanovy strom: %u\n",alloc_mem);

 free_mem=0; // moze dojst k uvolneniu prazdnych listov

 node_level=0;
 tri_add(oct_tree); // rekurzivne!!! zaradenie trojuholnikov do listov oktanoveho stromu (na najnizsiu hlbku)

 // INFO: uvolnena pamat z dovodu prazdnych listov
 //fprintf(stderr,"Velkost pamate uvolnenej prazdnymi listami: %u\n",free_mem);

 return(0);
}

// rekurzivne pridanie listov do oktanoveho stromu *************************
int mOcttree::node_add(_tree_node *node) // rekurzivna funkcia!!!
{
 // ak sa dosiahne hlbka TREE_SIZE, rekurzia pokracuje na predoslej hlbke
 if (node_level==TREE_SIZE) { node_level--; return(0); }

 // 4 predne podlisty (kocky) ----------------------------------------------

 // lava dolna kocka
 node->node[0]=new _tree_node; alloc_mem+=sizeof(_tree_node);
 if (node->node[0]==NULL) {
  fprintf(stderr,"Chyba vyhradenia pamate pre list: node0!\n"); return(1);
 }

 // nastavenie rozmerov podlistu (kocky)
 node->node[0]->minx=node->minx; node->node[0]->maxx=node->minx+(node->maxx-node->minx)/2.0f;
 node->node[0]->miny=node->miny; node->node[0]->maxy=node->miny+(node->maxy-node->miny)/2.0f;
 node->node[0]->minz=node->minz; node->node[0]->maxz=node->minz+(node->maxz-node->minz)/2.0f;
 // INFO: pridanie podlistu
 //fprintf(stderr,"front left down\n");

 // identifikator listu
 node->node[0]->id=nodes_num;
 nodes_num++;
 for (a=0;a<8;a++) node->node[0]->node[a]=NULL; // vsetky podlisty budu zatial prazdne
 node->node[0]->node_tri=NULL; // list zatial nema ziadne trojuholniky
 node->node[0]->node_tri_num=0;
 node->node[0]->node_draw_bit=true; // nastavenie indikatora zobrazenia listu

 node_level++; // nova hlbka
 // INFO: aktualna hlbka oktanoveho stromu
 //fprintf(stderr,"node_level: %u\n",node_level);
 node_add(node->node[0]); // REKURZIA

 // prava dolna kocka
 node->node[1]=new _tree_node; alloc_mem+=sizeof(_tree_node);
 if (node->node[1]==NULL) {
  fprintf(stderr,"Chyba vyhradenia pamate pre list: node1!\n"); return(1);
 }

 // nastavenie rozmerov podlistu (kocky)
 node->node[1]->minx=node->minx+(node->maxx-node->minx)/2.0f; node->node[1]->maxx=node->maxx;
 node->node[1]->miny=node->miny;                              node->node[1]->maxy=node->miny+(node->maxy-node->miny)/2.0f;
 node->node[1]->minz=node->minz;                              node->node[1]->maxz=node->minz+(node->maxz-node->minz)/2.0f;
 // INFO: pridanie podlistu
 //fprintf(stderr,"front right down\n");

 // identifikator listu
 node->node[1]->id=nodes_num;
 nodes_num++;
 for (a=0;a<8;a++) node->node[1]->node[a]=NULL; // vsetky podlisty budu zatial prazdne
 node->node[1]->node_tri=NULL; // list zatial nema ziadne trojuholniky
 node->node[1]->node_tri_num=0;
 node->node[1]->node_draw_bit=true; // nastavenie indikatora zobrazenia listu

 node_level++; // nova hlbka
 // INFO: aktualna hlbka oktanoveho stromu
 //fprintf(stderr,"node_level: %u\n",node_level);
 node_add(node->node[1]); // REKURZIA

 // prava horna kocka
 node->node[2]=new _tree_node; alloc_mem+=sizeof(_tree_node);
 if (node->node[2]==NULL) {
  fprintf(stderr,"Chyba vyhradenia pamate pre list: node2!\n"); return(1);
 }

 // nastavenie rozmerov sublistu (kocky)
 node->node[2]->minx=node->minx+(node->maxx-node->minx)/2.0f; node->node[2]->maxx=node->maxx;
 node->node[2]->miny=node->miny+(node->maxy-node->miny)/2.0f; node->node[2]->maxy=node->maxy;
 node->node[2]->minz=node->minz;                              node->node[2]->maxz=node->minz+(node->maxz-node->minz)/2.0f;
 // INFO: pridanie podlistu
 //fprintf(stderr,"front right up\n");

 // identifikator listu
 node->node[2]->id=nodes_num;
 nodes_num++;
 for (a=0;a<8;a++) node->node[2]->node[a]=NULL; // vsetky podlisty budu zatial prazdne
 node->node[2]->node_tri=NULL; // list zatial nema ziadne trojuholniky
 node->node[2]->node_tri_num=0;
 node->node[2]->node_draw_bit=true; // nastavenie indikatora zobrazenia listu

 node_level++; // nova hlbka
 // INFO: aktualna hlbka oktanoveho stromu
 //fprintf(stderr,"node_level: %u\n",node_level);
 node_add(node->node[2]); // REKURZIA

 // lava horna kocka
 node->node[3]=new _tree_node; alloc_mem+=sizeof(_tree_node);
 if (node->node[3]==NULL) {
  fprintf(stderr,"Chyba vyhradenia pamate pre list: node3!\n"); return(1);
 }

 // nastavenie rozmerov sublistu (kocky)
 node->node[3]->minx=node->minx;                              node->node[3]->maxx=node->minx+(node->maxx-node->minx)/2.0f;
 node->node[3]->miny=node->miny+(node->maxy-node->miny)/2.0f; node->node[3]->maxy=node->maxy;
 node->node[3]->minz=node->minz;                              node->node[3]->maxz=node->minz+(node->maxz-node->minz)/2.0f;
 // INFO: pridanie podlistu
 //fprintf(stderr,"front left up\n");

 // identifikator listu
 node->node[3]->id=nodes_num;
 nodes_num++;
 for (a=0;a<8;a++) node->node[3]->node[a]=NULL; // vsetky podlisty budu zatial prazdne
 node->node[3]->node_tri=NULL; // list zatial nema ziadne trojuholniky
 node->node[3]->node_tri_num=0;
 node->node[3]->node_draw_bit=true; // nastavenie indikatora zobrazenia listu

 node_level++; // nova hlbka
 // INFO: aktualna hlbka oktanoveho stromu
 //fprintf(stderr,"node_level: %u\n",node_level);
 node_add(node->node[3]); // REKURZIA

 // 4 zadne podlisty (kocky) -----------------------------------------------

 // lava dolna kocka
 node->node[4]=new _tree_node; alloc_mem+=sizeof(_tree_node);
 if (node->node[4]==NULL) {
  fprintf(stderr,"Chyba vyhradenia pamate pre list: node4!\n"); return(1);
 }

 // nastavenie rozmerov podlistu (kocky)
 node->node[4]->minx=node->minx;                              node->node[4]->maxx=node->minx+(node->maxx-node->minx)/2.0f;
 node->node[4]->miny=node->miny;                              node->node[4]->maxy=node->miny+(node->maxy-node->miny)/2.0f;
 node->node[4]->minz=node->minz+(node->maxz-node->minz)/2.0f; node->node[4]->maxz=node->maxz;
 // INFO: pridanie podlistu
 //fprintf(stderr,"back left down\n");

 // identifikator listu
 node->node[4]->id=nodes_num;
 nodes_num++;
 for (a=0;a<8;a++) node->node[4]->node[a]=NULL; // vsetky podlisty budu zatial prazdne
 node->node[4]->node_tri=NULL; // list zatial nema ziadne trojuholniky
 node->node[4]->node_tri_num=0;
 node->node[4]->node_draw_bit=true; // nastavenie indikatora zobrazenia listu

 node_level++; // nova hlbka
 // INFO: aktualna hlbka oktanoveho stromu
 //fprintf(stderr,"node_level: %u\n",node_level);
 node_add(node->node[4]); // REKURZIA

 // prava dolna kocka
 node->node[5]=new _tree_node; alloc_mem+=sizeof(_tree_node);
 if (node->node[5]==NULL) {
  fprintf(stderr,"Chyba vyhradenia pamate pre list: node5!\n"); return(1);
 }

 // nastavenie rozmerov podlistu (kocky)
 node->node[5]->minx=node->minx+(node->maxx-node->minx)/2.0f; node->node[5]->maxx=node->maxx;
 node->node[5]->miny=node->miny;                              node->node[5]->maxy=node->miny+(node->maxy-node->miny)/2.0f;
 node->node[5]->minz=node->minz+(node->maxz-node->minz)/2.0f; node->node[5]->maxz=node->maxz;
 // INFO: pridanie podlistu
 //fprintf(stderr,"front right down\n");

 // identifikator listu
 node->node[5]->id=nodes_num;
 nodes_num++;
 for (a=0;a<8;a++) node->node[5]->node[a]=NULL; // vsetky podlisty budu zatial prazdne
 node->node[5]->node_tri=NULL; // list zatial nema ziadne trojuholniky
 node->node[5]->node_tri_num=0;
 node->node[5]->node_draw_bit=true; // nastavenie indikatora zobrazenia listu

 node_level++; // nova hlbka
 // INFO: aktualna hlbka oktanoveho stromu
 //fprintf(stderr,"node_level: %u\n",node_level);
 node_add(node->node[5]); // REKURZIA

 // prava horna kocka
 node->node[6]=new _tree_node; alloc_mem+=sizeof(_tree_node);
 if (node->node[6]==NULL) {
  fprintf(stderr,"Chyba vyhradenia pamate pre list: node6!\n"); return(1);
 }

 // nastavenie rozmerov sublistu (kocky)
 node->node[6]->minx=node->minx+(node->maxx-node->minx)/2.0f; node->node[6]->maxx=node->maxx;
 node->node[6]->miny=node->miny+(node->maxy-node->miny)/2.0f; node->node[6]->maxy=node->maxy;
 node->node[6]->minz=node->minz+(node->maxz-node->minz)/2.0f; node->node[6]->maxz=node->maxz;
 // INFO: pridanie podlistu
 //fprintf(stderr,"front right up\n");

 // identifikator listu
 node->node[6]->id=nodes_num;
 nodes_num++;
 for (a=0;a<8;a++) node->node[6]->node[a]=NULL; // vsetky podlisty budu zatial prazdne
 node->node[6]->node_tri=NULL; // list zatial nema ziadne trojuholniky
 node->node[6]->node_tri_num=0;
 node->node[6]->node_draw_bit=true; // nastavenie indikatora zobrazenia listu

 node_level++; // nova hlbka
 // INFO: aktualna hlbka oktanoveho stromu
 //fprintf(stderr,"node_level: %u\n",node_level);
 node_add(node->node[6]); // REKURZIA

 // lava horna kocka
 node->node[7]=new _tree_node; alloc_mem+=sizeof(_tree_node);
 if (node->node[7]==NULL) {
  fprintf(stderr,"Chyba vyhradenia pamate pre list: node7!\n"); return(1);
 }

 // nastavenie rozmerov sublistu (kocky)
 node->node[7]->minx=node->minx;                              node->node[7]->maxx=node->minx+(node->maxx-node->minx)/2.0f;
 node->node[7]->miny=node->miny+(node->maxy-node->miny)/2.0f; node->node[7]->maxy=node->maxy;
 node->node[7]->minz=node->minz+(node->maxz-node->minz)/2.0f; node->node[7]->maxz=node->maxz;
 // INFO: pridanie podlistu
 //fprintf(stderr,"front left up\n");

 // identifikator listu
 node->node[7]->id=nodes_num;
 nodes_num++;
 for (a=0;a<8;a++) node->node[7]->node[a]=NULL; // vsetky podlisty budu zatial prazdne
 node->node[7]->node_tri=NULL; // list zatial nema ziadne trojuholniky
 node->node[7]->node_tri_num=0;
 node->node[7]->node_draw_bit=true; // nastavenie indikatora zobrazenia listu

 node_level++; // nova hlbka
 // INFO: aktualna hlbka oktanoveho stromu
 //fprintf(stderr,"node_level: %u\n",node_level);
 node_add(node->node[7]); // REKURZIA

 // rekurzia pokracuje na predoslej hlbke
 node_level--; return(0);
}

// zaradenie trojuholnikov do listov oktanoveho stromu *********************
int mOcttree::tri_add(_tree_node *node) // rekurzivna funkcia!!!
{
 if (node_level==TREE_SIZE) { // pridava sa iba na najnizsiu hlbku
  node->node_tri_num=0; // spocitanie poctu trojuholnikov listu

  // zistenie velkosti pamate, ktoru je potrebne vyhradit
  // pre pole trojuholnikov priradenych listu
  tmp_tri=first_tri;
  for (a=0;a<tri_num;a++) { // pre vsetky trojuholniky z triedy mTriangle
   b=0;
   // zistenie, ci je trojuholnik v liste (kocke)
   // test x hodnot trojuholnika
   if ((tmp_tri->minx>=node->minx)&&(tmp_tri->minx<=node->maxx)) b=1;
   if ((tmp_tri->maxx>=node->minx)&&(tmp_tri->maxx<=node->maxx)) b=1;
   // test y hodnot vrcholov
   if (b==1) {
    if ((tmp_tri->miny>=node->miny)&&(tmp_tri->miny<=node->maxy)) b=2;
    if ((tmp_tri->maxy>=node->miny)&&(tmp_tri->maxy<=node->maxy)) b=2;
   }
   // test z hodnot vrcholov
   if (b==2) {
    if ((tmp_tri->minz>=node->minz)&&(tmp_tri->minz<=node->maxz)) b=3;
    if ((tmp_tri->maxz>=node->minz)&&(tmp_tri->maxz<=node->maxz)) b=3;
   }
   // ak trojuholnik aspon ciastocne zasahuje do listu (kocky),
   // zvysi sa pocet trojuholnikov listu
   if (b==3) node->node_tri_num++;

   // nasledujuci trojuholnik v zozname
   tmp_tri=tmp_tri->next;
  }

  // INFO: pocet trojuholnikov listu
  //fprintf(stderr,"node_tri_num: %u\n",node->node_tri_num);

  if (node->node_tri_num!=0) { // v liste budu trojuholniky

   // INFO: identifikator listu s trojuholnikmi
   //fprintf(stderr,"%u ",node->id);

   // vyhradenie pamate pre trojuholniky listu
   node->node_tri=(_tri **) malloc(node->node_tri_num*sizeof(_tri));
   if (node->node_tri==NULL) {
    fprintf(stderr,"Chyba vyhradenia pamate pre trojuholniky listu!\n"); return(1);
   }

   // zvysenie celkoveho poctu trojuholnikov oktanoveho stromu
   // tato hodnota je potrebna neskor a to pre spravne vytvorenie
   // pola odkazov na aktivne trojuholniky v triede mTriangle
   // pomocou funkcie tri_finish();
   octtree_tri_num+=node->node_tri_num;

   // naplnenie pola odkazov na trojuholniky priradene listu
   tmp_tri=first_tri;
   c=0;
   for (a=0;a<tri_num;a++) {
    b=0;
    // zistenie, ci je trojuholnik v liste (kocke)
    // test x hodnot trojuholnika
    if ((tmp_tri->minx>=node->minx)&&(tmp_tri->minx<=node->maxx)) b=1;
    if ((tmp_tri->maxx>=node->minx)&&(tmp_tri->maxx<=node->maxx)) b=1;
    // test y hodnot vrcholov
    if (b==1) {
     if ((tmp_tri->miny>=node->miny)&&(tmp_tri->miny<=node->maxy)) b=2;
     if ((tmp_tri->maxy>=node->miny)&&(tmp_tri->maxy<=node->maxy)) b=2;
    }
    // test z hodnot vrcholov
    if (b==2) {
     if ((tmp_tri->minz>=node->minz)&&(tmp_tri->minz<=node->maxz)) b=3;
     if ((tmp_tri->maxz>=node->minz)&&(tmp_tri->maxz<=node->maxz)) b=3;
    }
    // ak je trojuholnik v liste, odkaz na tento trojuholnik
    // sa umiestni do pola odkazov na trojuholniky priradene listu
    if (b==3) { node->node_tri[c]=tmp_tri; c++; }

    // nasledujuci trojuholnik v zozname
    tmp_tri=tmp_tri->next;
   }
  }
  else { node_level--; return(1); } // list je potrebne vymazat

  // rekurzia pokracuje na predoslej hlbke
  node_level--; return(0);
 } // pridava sa iba na najnizsiu hlbku

 node_level++;
 if (tri_add(node->node[0])) {
  // list sa vymaze, nakolko v nom nie su ziadne trojuholniky
  if (node->node[0]->node_tri!=NULL) free(node->node[0]->node_tri); // zrusenie pola trojuholnikov
  delete node->node[0]; node->node[0]=NULL; // zrusenie listu
  free_mem+=sizeof(_tree_node);
 }
 node_level++;
 if (tri_add(node->node[1])) {
  // list sa vymaze, nakolko v nom nie su ziadne trojuholniky
  if (node->node[1]->node_tri!=NULL) free(node->node[1]->node_tri); // zrusenie pola trojuholnikov
  delete node->node[1]; node->node[1]=NULL; // zrusenie listu
  free_mem+=sizeof(_tree_node);
 }
 node_level++;
 if (tri_add(node->node[2])) {
  // list sa vymaze, nakolko v nom nie su ziadne trojuholniky
  if (node->node[2]->node_tri!=NULL) free(node->node[2]->node_tri); // zrusenie pola trojuholnikov
  delete node->node[2]; node->node[2]=NULL; // zrusenie listu
  free_mem+=sizeof(_tree_node);
 }
 node_level++;
 if (tri_add(node->node[3])) {
  // list sa vymaze, nakolko v nom nie su ziadne trojuholniky
  if (node->node[3]->node_tri!=NULL) free(node->node[3]->node_tri); // zrusenie pola trojuholnikov
  delete node->node[3]; node->node[3]=NULL; // zrusenie listu
  free_mem+=sizeof(_tree_node);
 }
 node_level++;
 if (tri_add(node->node[4])) {
  // list sa vymaze, nakolko v nom nie su ziadne trojuholniky
  if (node->node[4]->node_tri!=NULL) free(node->node[4]->node_tri); // zrusenie pola trojuholnikov
  delete node->node[4]; node->node[4]=NULL; // zrusenie listu
  free_mem+=sizeof(_tree_node);
 }
 node_level++;
 if (tri_add(node->node[5])) {
  // list sa vymaze, nakolko v nom nie su ziadne trojuholniky
  if (node->node[5]->node_tri!=NULL) free(node->node[5]->node_tri); // zrusenie pola trojuholnikov
  delete node->node[5]; node->node[5]=NULL; // zrusenie listu
  free_mem+=sizeof(_tree_node);
 }
 node_level++;
 if (tri_add(node->node[6])) {
  // list sa vymaze, nakolko v nom nie su ziadne trojuholniky
  if (node->node[6]->node_tri!=NULL) free(node->node[6]->node_tri); // zrusenie pola trojuholnikov
  delete node->node[6]; node->node[6]=NULL; // zrusenie listu
  free_mem+=sizeof(_tree_node);
 }
 node_level++;
 if (tri_add(node->node[7])) {
  // list sa vymaze, nakolko v nom nie su ziadne trojuholniky
  if (node->node[7]->node_tri!=NULL) free(node->node[7]->node_tri); // zrusenie pola trojuholnikov
  delete node->node[7]; node->node[7]=NULL; // zrusenie listu
  free_mem+=sizeof(_tree_node);
 }

 // rekurzia pokracuje na predoslej hlbke
 node_level--; return(0);
}

// vykreslenie oktanoveho stromu *******************************************
void mOcttree::Octtree_view(void)
{
 glPushAttrib(GL_LIGHTING_BIT|GL_DEPTH_BUFFER_BIT);
 glDisable(GL_LIGHTING);   // vypnutie osvetlenia
 glDisable(GL_DEPTH_TEST); // vypnutie testu hlbky

 glColor4f(1.0f,1.0f,0.0f,1.0f); // zlta farba

 node_level=0;
 node_view(oct_tree); // rekurzivne!!! vykreslenie hran listov (kociek)

 glColor4f(1.0f,1.0f,1.0f,1.0f); // biela farba

 glPopAttrib(); // vyber ulozenych atributov (LIGHTING,DEPTH_BUFFER)
}

// rekurzivne vykreslenie listov oktanoveho stromu *************************
int mOcttree::node_view(_tree_node *node) // rekurzivna funkcia!!!
{
 if (node!=NULL) {
  if (node->node_draw_bit) { // ak sa ma list (kocka) kreslit
   // predna stena
   glBegin(GL_LINE_LOOP);
    glVertex3f(node->minx,node->miny,node->minz);
    glVertex3f(node->maxx,node->miny,node->minz);
    glVertex3f(node->maxx,node->maxy,node->minz);
    glVertex3f(node->minx,node->maxy,node->minz);
   glEnd();
   // zadna stena
   glBegin(GL_LINE_LOOP);
    glVertex3f(node->minx,node->miny,node->maxz);
    glVertex3f(node->maxx,node->miny,node->maxz);
    glVertex3f(node->maxx,node->maxy,node->maxz);
    glVertex3f(node->minx,node->maxy,node->maxz);
   glEnd();
   // spojovacie ciary
   glBegin(GL_LINES);
    // lava dolna
    glVertex3f(node->minx,node->miny,node->minz);
    glVertex3f(node->minx,node->miny,node->maxz);
    // prava dolna
    glVertex3f(node->maxx,node->miny,node->minz);
    glVertex3f(node->maxx,node->miny,node->maxz);
    // prava horna
    glVertex3f(node->maxx,node->maxy,node->minz);
    glVertex3f(node->maxx,node->maxy,node->maxz);
    // lava horna
    glVertex3f(node->minx,node->maxy,node->minz);
    glVertex3f(node->minx,node->maxy,node->maxz);
   glEnd();
  }
 }

 // ak sa dosiahne hlbka TREE_SIZE, rekurzia pokracuje na predoslej hlbke
 if (node_level==TREE_SIZE) { node_level--; return(0); }

 node_level++; node_view(node->node[0]);
 node_level++; node_view(node->node[1]);
 node_level++; node_view(node->node[2]);
 node_level++; node_view(node->node[3]);
 node_level++; node_view(node->node[4]);
 node_level++; node_view(node->node[5]);
 node_level++; node_view(node->node[6]);
 node_level++; node_view(node->node[7]);

 // rekurzia pokracuje na predoslej hlbke
 node_level--; return(0);
}

// zrusenie oktanoveho stromu **********************************************
int mOcttree::Octtree_destroy(void)
{
 // zacatie spocitavania velkosti pamate, ktoru uvolni oktanovy strom
 free_mem=0;

 // zrusenie rodicovskeho listu oktanoveho stromu
 if (oct_tree!=NULL) {

  // rekurzivne!!! zrusenie vsetkych podlistov danej hlbky
  for (a=TREE_SIZE;a>0;a--) {
   del_level=a;
   node_level=0;
   node_del(oct_tree);
  }

  // zrusenie pola trojuholnikov rodicovskeho listu
  if (oct_tree->node_tri!=NULL) free(oct_tree->node_tri);
  // zrusenie rodicovskeho listu
  delete oct_tree; oct_tree=NULL;
  free_mem+=sizeof(_tree_node);
 }

 // INFO: uvolnena pamat
 //fprintf(stderr,"Velkost pamate uvolnenej oktanovym stromom: %u\n",free_mem);

 nodes_num=0; // pocet listov oktanoveho stromu

 octtree_tri_num=0; // celkovy pocet trojuholnikov oktanoveho stromu

 return(0);
}

// rekurzivne zrusenie listov oktanoveho stromu ****************************
int mOcttree::node_del(_tree_node *node) // rekurzivna funkcia!!!
{
 if ((node_level==del_level)&&(node!=NULL)) { // rusi sa v konkretnej hlbke
  // zrusenie pola trojuholnikov
  if (node->node_tri!=NULL) free(node->node_tri);
  // zrusenie listu
  delete node; node=NULL;
  free_mem+=sizeof(_tree_node);
  // ak sa dosiahne patricna hlbka, rekurzia pokracuje na predoslej hlbke
  node_level--; return(0);
 }

 // ak sa dosiahne hlbka TREE_SIZE, rekurzia pokracuje na predoslej hlbke
 if (node_level==TREE_SIZE) { node_level--; return(0); }

 node_level++; node_del(node->node[0]);
 node_level++; node_del(node->node[1]);
 node_level++; node_del(node->node[2]);
 node_level++; node_del(node->node[3]);
 node_level++; node_del(node->node[4]);
 node_level++; node_del(node->node[5]);
 node_level++; node_del(node->node[6]);
 node_level++; node_del(node->node[7]);

 // rekurzia pokracuje na predoslej hlbke
 node_level--; return(0);
}

// vyber trojuholnikov z oktanoveho stromu *********************************
void mOcttree::Octtree_finish(mTriangle *T,mFrustum *F,GLfloat AX,GLfloat AY,GLfloat AZ)
{
 // aktivne trojuholniky z triedy mTriangle
 act_tri=T->act_tri;
 act_tri_num=0;

 // pocet testov na viditelnost listov oktanoveho stromu
 tests_num=0;

 // Frustum z triedy mFrustum
 OGLFrustum=F;

 // poloha AVATARa
 avatar_x=AX; avatar_y=AY; avatar_z=AZ;

 // rekurzivny!!! vyber aktivnych trojuholnikov z listov oktanoveho stromu
 node_level=0;
 node_finish(oct_tree);

 // nastavenie poctu aktivnych trojuholnikov v triede mTriangle
 T->act_tri_num=act_tri_num;
}

// rekurzivny vyber aktivnych trojuholnikov z listov oktanoveho stromu *****
int mOcttree::node_finish(_tree_node *node) // rekurzivna funkcia!!!
{
 // pocet testov s oktanovym stromom
 tests_num++;

 if (node!=NULL) {

  // Frustum Culling pre listy oktanoveho stromu
  if (OGLFrustum->CubeInFrustum2(node->minx,node->maxx,
                                 node->miny,node->maxy,
                                 node->minz,node->maxz)) {
   // list je viditelny
   node->node_draw_bit=true;

   //if ((avatar_x>=node->minx)&&(avatar_x<=node->maxx)&&
   //    (avatar_y>=node->miny)&&(avatar_y<=node->maxy)&&
   //    (avatar_z>=node->minz)&&(avatar_z<=node->maxz)) {

    // vsetky odkazy ulozene v poli odkazov listu
    // sa umiestnia do pola odkazov v triede mTriangle,
    // trojuholnikom sa nastavi indikator spracovania
    // v pripade vyberu listu s nulovym poctom trojuholnikov
    // sa nebude vykonavat presun odkazov takze sa
    // nemusi testovat na poslednu hlbku, v ktorej
    // sa nachadzaju trojuholniky
    for (a=0;a<node->node_tri_num;a++) {
     act_tri[act_tri_num]=node->node_tri[a];
     act_tri[act_tri_num]->finish_bit=false;
     act_tri_num++;
    }
   //}

  } // Frustum Culling pre listy oktanoveho stromu
  else {
   // list nie je viditelny a ani jeden z jeho podlistov
   // nie je viditelny, takto sa jednoduchym testom
   // odstrani velke mnozstvo trojuholnikov
   node->node_draw_bit=false;

   // rekurzia pokracuje na predoslej hlbke
   node_level--; return(0);
  }

 }

 // ak sa dosiahne hlbka TREE_SIZE, rekurzia pokracuje na predoslej hlbke
 if (node_level==TREE_SIZE) { node_level--; return(0); }

 node_level++; node_finish(node->node[0]);
 node_level++; node_finish(node->node[1]);
 node_level++; node_finish(node->node[2]);
 node_level++; node_finish(node->node[3]);
 node_level++; node_finish(node->node[4]);
 node_level++; node_finish(node->node[5]);
 node_level++; node_finish(node->node[6]);
 node_level++; node_finish(node->node[7]);

 // rekurzia pokracuje na predoslej hlbke
 node_level--; return(0);
}
