#ifndef _MOCTTREE_H_
#define _MOCTTREE_H_

// trojuholnik
#include "../mFrontend.h"

typedef struct __tree_node {            // list oktanoveho stromu
 GLuint id;                             // identifikator listu oktanoveho stromu

 GLfloat minx,maxx,miny,maxy,minz,maxz; // steny listu (kocky) oktanoveho stromu
 __tree_node *node[8];                  // odkazy na 8 podlistov listu

 _tri **node_tri;                       // pole odkazov na trojuholniky priradene listu
 GLuint node_tri_num;                   // pocet trojuholnikov listu

 bool node_draw_bit;                    // indikator zobrazenia listu
} _tree_node;

// trieda premennych a funkcii (objektov) pre pracu s oktanovym stromom
class mOcttree {
public:
 mOcttree();          // konstruktor
 virtual ~mOcttree(); // destruktor

 GLuint octtree_tri_num; // celkovy pocet trojuholnikov oktanoveho stromu

 GLuint tests_num; // pocet testov na viditelnost listov oktanoveho stromu

 GLuint nodes_num; // pocet listov oktanoveho stromu

 // vytvorenie oktanoveho stromu, prevezme sa odkaz na trojuholniky z triedy mTriangle
 // a celkovy pocet trojuholnikov z triedy mTriangle
 int Octtree_create(mTriangle *T,GLuint N);
 // vykreslenie oktanoveho stromu
 void Octtree_view(void);
 // zrusenie oktanoveho stromu
 int Octtree_destroy(void);
 // vyber trojuholnikov z oktanoveho stromu a presun odkazov do triedy mTriangle
 void Octtree_finish(mTriangle *T,mFrustum *F,GLfloat AX,GLfloat AY,GLfloat AZ);
private:
 GLuint TREE_SIZE; // hlbka oktanoveho stromu

 _tree_node *oct_tree; // rodicovsky list oktanoveho stromu
 //GLuint nodes_num;     // pocet listov oktanoveho stromu

 _tri *first_tri,*tmp_tri; // trojuholniky z triedy mTriangle
 GLuint tri_num;           // celkovy pocet trojuholnikov z triedy mTriangle
 _tri **act_tri;           // aktivne trojuholniky z triedy mTriangle
 GLuint act_tri_num;       // pocet aktivnych trojuholnikov z triedy mTriangle

 mFrustum *OGLFrustum; // Frustum z triedy mFrustum

 GLfloat avatar_x,avatar_y,avatar_z; // poloha AVATARa

 // pocitadla vyhradenej/uvolnenej pamate
 GLuint alloc_mem;
 GLuint free_mem;

 GLuint a,b,c; // pomocne premenne

 GLuint node_level; // hlbka oktanoveho stromu
 GLuint del_level;  // hlbka oktanoveho stromu, v ktorej budu zmazane listy

 // rekurzivne!!! pridanie listov do oktanoveho stromu
 int node_add(_tree_node *node);
 // rekurzivne!!! zaradenie trojuholnikov do listov oktanoveho stromu (na najnizsiu hlbku)
 int tri_add(_tree_node *node);
 // rekurzivne!!! vykreslenie listov oktanoveho stromu
 int node_view(_tree_node *node);
 // rekurzivne!!! zrusenie listov oktanoveho stromu
 int node_del(_tree_node *node);

 // rekurzivny!!! vyber aktivnych trojuholnikov z listov oktanoveho stromu
 int node_finish(_tree_node *node);
};

#endif
