// hlavickove subory *******************************************************
// GL kniznica
#include <GL/gl.h>
// funkcie SDL_image kniznice
#include <SDL/SDL_image.h>

// vlastny .h subor
#include "mText.h"

// konstruktor *************************************************************
mText::mText()
{

}

// destruktor **************************************************************
mText::~mText()
{
 glDeleteTextures(1,&tex_id); // zrusenie objektu Textury
 glDeleteLists(256,list_id);  // zrusenie DisplayListu
}

// inicializacia textu *****************************************************
int mText::t_initialize(const char *FONT_FILE,const unsigned int W, const unsigned int H)
{
 // ulozenie sirky a vysky okna, do ktoreho sa bude vypisovat text,
 // zaroven sa tym nastavi aj velkost pisma
 scr_width=W; scr_height=H;

 // pociatocne nastavenie identifikatora Textury a DisplayListu
 tex_id=0; list_id=0;

 // SDL nacitanie suboru s texturou
 SDL_Surface *TextureImage;
 // nacitanie udajov o texture
 TextureImage=IMG_Load(FONT_FILE);
 if (TextureImage==NULL) {
  fprintf(stderr,"Nepodarilo sa nacitat texturu: %s\n",FONT_FILE);
  return(1);
 }

 // nastavenie sirky, vysky textury
 t_w=TextureImage->w; t_h=TextureImage->h;

 // premenna data bude odkazovat na farbu pixelov textury
 data=(unsigned char *) TextureImage->pixels;

 // OpenGL vygenerovanie objektu textury
 glGenTextures(1,&tex_id);
 glBindTexture(GL_TEXTURE_2D,tex_id);
 glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
 glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
 glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP);
 glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP);
 glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,t_w,t_h,0,GL_RGBA,GL_UNSIGNED_BYTE,data);

 // INFO: identifikator, sirka, vyska a nazov suboru textury
 fprintf(stderr,"Font textura: %i %i %i %s\n",tex_id,t_w,t_h,FONT_FILE);

 // rozmer jedneho znaku v texture
 step=t_w/16; // 512/16=32 = rozmer jedneho znaku (pozicie so znakom)
 // nastavenie odkazov na textury patriace jednotlivym znakom
 for (y=0,i=0;y<16;y++) {  // pre 16 pozicii v smere Y
  for (x=0;x<16;x++,i++) { // pre 16 pozicii v smere X
   chars[i][0]=0;
   // najdenie X-ovej suradnice, na ktorej zacina znak
   for (j=0;j<step;j++) {
    ptr=data+(y*t_w*step+x*step+j)*4;
    for (k=0;k<step;k++) {
     if (*(ptr+3)!=0) break;
     ptr+=t_w*4;
    }
    if (k!=step) break;
    chars[i][0]++; // postupuje sa zlava - doprava, hlada sa zaciatok znaku
   }
   chars[i][1]=0;
   // ak sa na danej pozicii nenasiel znak, pokracuje sa dalsou poziciou
   if (chars[i][0]==step) continue;
   // najdenie X-ovej suradnice, na ktorej konci znak
   for (j=step-1;j>=0;j--) {
    ptr=data+(y*t_w*step+x*step+j)*4;
    for (k=0;k<step;k++) {
     if (*(ptr+3)!=0) break;
     ptr+=t_w*4;
    }
    if (k!=step) break;
    chars[i][1]++; // postupuje sa zprava - dolava, hlada sa koniec znaku
   }
   // vypocet sirky znaku
   chars[i][1]=step-chars[i][0]-chars[i][1];
  }
 }

 // vygenerovanie 256tich DisplayListov
 list_id=glGenLists(256);

 // priradenie textur jednotlivym DisplayListom
 for (y=0,i=0;y<16;y++) {  // pre 16 pozicii Y
  for (x=0;x<16;x++,i++) { // pre 16 pozicii X
   // X suradnica textury
   s=(GLfloat) x/16.0f+(GLfloat) chars[i][0]/(GLfloat) t_w;
   // Y suradnica textury
   t=(GLfloat) y/16.0f;
   // sirka znaku
   ds=(GLfloat) chars[i][1]/(GLfloat) t_w;
   // vyska znaku = cela vyska jednej pozicie
   dt=1.0f/16.0f;
   // ulozenie znaku do DL na poziciu list_id+i
   glNewList(list_id+i,GL_COMPILE);
    glBegin(GL_QUADS);
     glTexCoord2f(s,t);       glVertex2i(0,0);
     glTexCoord2f(s,t+dt);    glVertex2i(0,step);
     glTexCoord2f(s+ds,t+dt); glVertex2i(chars[i][1],step);
     glTexCoord2f(s+ds,t);    glVertex2i(chars[i][1],0);
    glEnd();
    // posun znaku o jeho sirku v smere X = odsadenie medzi znakmi
    glTranslatef((GLfloat) chars[i][1],0.0f,0.0f);
   glEndList();
  }
 }

 // uvolnenie pouzitej pamate
 if (TextureImage!=NULL) SDL_FreeSurface(TextureImage);
 TextureImage=NULL;

 return(0);
}

// farba textu *************************************************************
void mText::t_color(GLfloat R,GLfloat G,GLfloat B,GLfloat A)
{
 color[0]=R; color[1]=G; color[2]=B; color[3]=A;
}

// zaciatok vypisu textu ***************************************************
void mText::t_start(void)
{
 // ulozenie potrebnych atributov
 glPushAttrib(GL_LIGHTING_BIT|GL_DEPTH_BUFFER_BIT|GL_TEXTURE_BIT);

 glMatrixMode(GL_PROJECTION);
 glPushMatrix(); // ulozenie projection matice
 glLoadIdentity();
 glOrtho(0,scr_width,scr_height,0,-1,1); // nastavenie pravouhleho premietania
 glMatrixMode(GL_MODELVIEW);
 glPushMatrix(); // ulozenie modelview matice
 glLoadIdentity();
 glDisable(GL_LIGHTING);   // vypnutie osvetlenia
 glDisable(GL_DEPTH_TEST); // vypnutie testu hlbky
 glDepthMask(GL_FALSE);
 glEnable(GL_BLEND);       // znaky su polopriesvitne textury
 glBlendFunc(GL_ONE,GL_ONE_MINUS_SRC_ALPHA);
 glEnable(GL_TEXTURE_2D);  // zapnutie texturovania
 glBindTexture(GL_TEXTURE_2D,tex_id);

 // nastavenie farby textu
 glColor4f(color[0],color[1],color[2],color[3]);
}

// koniec vypisu textu *****************************************************
void mText::t_end(void)
{
 // standardna farba - biela
 glColor4f(1.0f,1.0f,1.0f,1.0f);

 glDisable(GL_BLEND);
 glPopMatrix(); // vyber modelview matice
 glMatrixMode(GL_PROJECTION);
 glPopMatrix(); // vyber projection matice

 // aktualna musi byt vzdy modelview matica!
 glMatrixMode(GL_MODELVIEW);

 // vyber ulozenych atributov - LIGHTING, DEPTH_BUFFER, TEXTURE
 glPopAttrib();
}

// vypis textu *************************************************************
void mText::p_text(GLfloat X,GLfloat Y,const char *STRING)
{
 t_start();

 // vykreslenie znaku na poziciu X, Y
 glTranslatef(X,Y,0.0f);
 // vykreslenie DisplayListov prisluchajucich jednotlivym znakom
 for (l=0;*STRING!='\0';STRING++) {
  // z dovodu urychlenia sa nebude testovat na \n \r \t
  //if (*STRING=='\n') { // new line
  // l++;
  // glLoadIdentity();
  // glTranslatef(X,Y+(GLfloat) (step*l),0.0f);
  //} else if (*STRING=='\r') { // return
  //        glLoadIdentity();
  //        glTranslatef(X,Y+(GLfloat) (step*l),0.0f);
  //       } else if (*STRING=='\t') { // tabulator
  //               glTranslatef((GLfloat) step*2.0f,0.0f,0.0f);
  //              } else if (*STRING==' ') { // space
                       if (*STRING==' ') { // space
                        glTranslatef((GLfloat) step/2.0f,0.0f,0.0f);
                       } else { // znak
                          glCallList(list_id+*(unsigned char*) STRING);
                         }
 }

 t_end();
}

// vypis formatovaneho textu ***********************************************
void mText::p_ftext(GLfloat X,GLfloat Y,const char *FORMAT,...)
{
 va_list argptr;
 va_start(argptr,FORMAT);
 vsprintf(buf,FORMAT,argptr); // presun argumentov do pomocneho zasobnika
 va_end(argptr);              // na zaklade zvoleneho formatu vypis textu
 // vypis textu
 p_text(X,Y,buf);
}
