// hlavickove subory *******************************************************
// GL, GLU a GLEXT kniznice
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glext.h>
// funkcie SDL kniznice
#include <SDL/SDL.h>
#include <SDL/SDL_image.h>

// vlastny .h subor
#include "mTexture.h"

// konstruktor *************************************************************
mTexture::mTexture()
{
 texture_count=0;

 cubemap_id=0;
 crosshair_id=0;
 detail_tex_id=0;
}

// destruktor **************************************************************
mTexture::~mTexture()
{
 // odstranenie objektov textur
 for (t_a=0;t_a<texture_count;t_a++) glDeleteTextures(1,&texture_id[t_a]);
 // odstranenie texture objektu Cube Map textury
 if (cubemap_id) glDeleteTextures(1,&cubemap_id);
 // odstranenie texture objektu textury zameriavacieho kriza
 if (crosshair_id) glDeleteTextures(1,&crosshair_id);
 // odstranenie detailnej textury terenu
 if (detail_tex_id) glDeleteTextures(1,&detail_tex_id);
}

// nahratie 2D textury *****************************************************
int mTexture::Load2DTexture(GLuint ID,char *F,GLuint MODE)
{
 // SDL nacitanie suborov s texturami
 SDL_Surface *TextureImage;

 TextureImage=IMG_Load(F);

 // nahratie suboru s obrazkom
 if (TextureImage!=NULL) {
  // vygenerovanie textury
  glGenTextures(1,&texture_id[ID]);

  // INFO: identifikator a nazov suboru textury
  fprintf(stderr,"2D textura: %i %i %s\n",ID,texture_id[ID],F);

  // pripojenie textury
  glBindTexture(GL_TEXTURE_2D,texture_id[ID]);

  // nastavenie filteringu
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
  if (MODE&T_MIPMAP) {
   if (MODE&T_TRILINEAR) {
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
   } else {
      glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_NEAREST);
     }
  }

  // nastavenie opakovania sa textury
  if (MODE&T_CLAMP) {
   glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP);
   glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP);
  } else if (MODE&T_CLAMP_TO_EDGE) {
          glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP_TO_EDGE);
          glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP_TO_EDGE);
         } else {
            glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
            glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
           }

  // vytvorenie objektu textury
  if (MODE&T_MIPMAP) {
   // vytvorenie objektu MIPMAP textury
   gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB,TextureImage->w,TextureImage->h,
                     GL_BGR,GL_UNSIGNED_BYTE,TextureImage->pixels);
  } else {
     // vytvorenie objektu standardnej 2D textury
     glTexImage2D(GL_TEXTURE_2D,0,GL_RGB,TextureImage->w,TextureImage->h,
                  0,GL_BGR,GL_UNSIGNED_BYTE,TextureImage->pixels);
    }

  texture_count++; // zvysenie poctu nahratych textur
 }
 else return(1);

 // uvolnenie pouzitej pamate
 if (TextureImage) SDL_FreeSurface(TextureImage);
 TextureImage=NULL;

 return(0);
}

// nahratie 1D textury *****************************************************
int mTexture::Load1DTexture(GLuint ID,char *F,GLuint MODE)
{
 // SDL nacitanie suborov s texturami
 SDL_Surface *TextureImage;

 TextureImage=IMG_Load(F);

 // nahratie suboru s obrazkom
 if (TextureImage!=NULL) {
  // vygenerovanie textury
  glGenTextures(1,&texture_id[ID]);

  // INFO: identifikator a nazov suboru textury
  fprintf(stderr,"1D textura: %i %i %s\n",ID,texture_id[ID],F);

  // pripojenie textury
  glBindTexture(GL_TEXTURE_1D,texture_id[ID]);

  // nastavenie filteringu
  glTexParameteri(GL_TEXTURE_1D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
  glTexParameteri(GL_TEXTURE_1D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
  if (MODE&T_MIPMAP) {
   if (MODE&T_TRILINEAR) {
    glTexParameteri(GL_TEXTURE_1D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
   } else {
      glTexParameteri(GL_TEXTURE_1D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_NEAREST);
     }
  }

  // nastavenie opakovania sa textury
  if (MODE&T_CLAMP) {
   glTexParameteri(GL_TEXTURE_1D,GL_TEXTURE_WRAP_S,GL_CLAMP);
  } else if (MODE&T_CLAMP_TO_EDGE) {
          glTexParameteri(GL_TEXTURE_1D,GL_TEXTURE_WRAP_S,GL_CLAMP_TO_EDGE);
         } else {
            glTexParameteri(GL_TEXTURE_1D,GL_TEXTURE_WRAP_S,GL_REPEAT);
           }

  // vytvorenie objektu textury
  if (MODE&T_MIPMAP) {
   // vytvorenie objektu MIPMAP textury
   gluBuild1DMipmaps(GL_TEXTURE_1D,GL_RGB,TextureImage->w,
                     GL_BGR,GL_UNSIGNED_BYTE,TextureImage->pixels);
  } else {
     // vytvorenie objektu standardnej 1D textury
     glTexImage1D(GL_TEXTURE_1D,0,GL_RGB,TextureImage->w,0,
                  GL_BGR,GL_UNSIGNED_BYTE,TextureImage->pixels);
    }

  texture_count++; // zvysenie poctu nahratych textur
 }
 else return(1);

 // uvolnenie pouzitej pamate
 if (TextureImage) SDL_FreeSurface(TextureImage);
 TextureImage=NULL;

 return(0);
}

// nahratie Bump textury ***************************************************
int mTexture::LoadBumpTexture(GLuint ID1,GLuint ID2,char *F,GLuint MODE)
{
 // SDL nacitanie suborov s texturami
 SDL_Surface *TextureImage;

 TextureImage=IMG_Load(F);

 // nahratie suboru s obrazkom
 if (TextureImage!=NULL) {
  glPixelTransferf(GL_RED_SCALE,  0.5f); // 50% intenzita
  glPixelTransferf(GL_GREEN_SCALE,0.5f);
  glPixelTransferf(GL_BLUE_SCALE, 0.5f);

  // vygenerovanie textury
  glGenTextures(1,&texture_id[ID1]);

  // INFO: identifikator a nazov suboru textury
  fprintf(stderr,"Bump textura 1: %i %i %s\n",ID1,texture_id[ID1],F);

  // pripojenie textury
  glBindTexture(GL_TEXTURE_2D,texture_id[ID1]);

  // nastavenie filteringu
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
  if (MODE&T_MIPMAP) {
   if (MODE&T_TRILINEAR) {
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
   } else {
      glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_NEAREST);
     }
  }

  // nastavenie opakovania sa textury
  if (MODE&T_CLAMP) {
   glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP);
   glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP);
  } else if (MODE&T_CLAMP_TO_EDGE) {
          glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP_TO_EDGE);
          glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP_TO_EDGE);
         } else {
            glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
            glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
           }

  // vytvorenie objektu textury
  if (MODE&T_MIPMAP) {
   // vytvorenie objektu MIPMAP textury
   gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB8,TextureImage->w,TextureImage->h,
                     GL_BGR,GL_UNSIGNED_BYTE,TextureImage->pixels);
  } else {
     // vytvorenie objektu standardnej 2D textury
     glTexImage2D(GL_TEXTURE_2D,0,GL_RGB8,TextureImage->w,TextureImage->h,
                  0,GL_BGR,GL_UNSIGNED_BYTE,TextureImage->pixels);
    }

  // vytvorenie invertovanej textury
  td=(unsigned char *) TextureImage->pixels;
  for (ti=0;ti<3*TextureImage->w*TextureImage->h;ti++) td[ti]=255-td[ti];

  // vygenerovanie textury
  glGenTextures(1,&texture_id[ID2]);

  // INFO: identifikator a nazov suboru textury
  fprintf(stderr,"Bump textura 2: %i %i %s\n",ID2,texture_id[ID2],F);

  // pripojenie textury
  glBindTexture(GL_TEXTURE_2D,texture_id[ID2]);

  // nastavenie filteringu
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
  if (MODE&T_MIPMAP) {
   if (MODE&T_TRILINEAR) {
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
   } else {
      glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_NEAREST);
     }
  }

  // nastavenie opakovania sa textury
  if (MODE&T_CLAMP) {
   glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP);
   glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP);
  } else if (MODE&T_CLAMP_TO_EDGE) {
          glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP_TO_EDGE);
          glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP_TO_EDGE);
         } else {
            glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
            glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
           }

  // vytvorenie objektu textury
  if (MODE&T_MIPMAP) {
   // vytvorenie objektu MIPMAP textury
   gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB8,TextureImage->w,TextureImage->h,
                     GL_BGR,GL_UNSIGNED_BYTE,td);
  } else {
     // vytvorenie objektu standardnej 2D textury
     glTexImage2D(GL_TEXTURE_2D,0,GL_RGB8,TextureImage->w,TextureImage->h,
                  0,GL_BGR,GL_UNSIGNED_BYTE,td);
    }

  texture_count+=2; // zvysenie poctu nahratych textur

  glPixelTransferf(GL_RED_SCALE,  1.0f); // 100% intenzita
  glPixelTransferf(GL_GREEN_SCALE,1.0f);
  glPixelTransferf(GL_BLUE_SCALE, 1.0f);
 }
 else return(1);

 // uvolnenie pouzitej pamate
 if (TextureImage) SDL_FreeSurface(TextureImage);
 TextureImage=NULL;

 return(0);
}

// nahratie Cube Map textury ***********************************************
int mTexture::LoadCubeMapTexture(char *F1,char *F2,char *F3,
                                 char *F4,char *F5,char *F6,GLuint MODE)
{
 // SDL nacitanie suborov s texturami
 SDL_Surface *TextureImage;

 // vygenerovanie textury
 glGenTextures(1,&cubemap_id);

 // INFO: identifikator a nazov suboru textury
 fprintf(stderr,"Cube Map textura: %i\n",cubemap_id);

 // pripojenie textury
 glBindTexture(GL_TEXTURE_CUBE_MAP_ARB,cubemap_id);
 // nastavenie filteringu
 glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
 glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
 // nastavenie opakovania sa textury
 if (MODE&T_CLAMP_TO_EDGE) {
  glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB,GL_TEXTURE_WRAP_S,GL_CLAMP_TO_EDGE);
  glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB,GL_TEXTURE_WRAP_T,GL_CLAMP_TO_EDGE);
 } else {
    glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB,GL_TEXTURE_WRAP_S,GL_REPEAT);
    glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB,GL_TEXTURE_WRAP_T,GL_REPEAT);
   }

 // vlozenie obrazkov do textury
 for (t_a=0;t_a<6;t_a++) {
  switch(t_a) {
   case 0: TextureImage=IMG_Load(F1);
           if (TextureImage==NULL) return(0);
           // INFO: identifikator a nazov suboru textury
           fprintf(stderr,"CM textura: %i %s\n",t_a,F1);
           // vytvorenie objektu textury
           glTexImage2D(GL_TEXTURE_CUBE_MAP_NEGATIVE_X_ARB,0,GL_RGB,
                        TextureImage->w,TextureImage->h,0,
                        GL_BGR,GL_UNSIGNED_BYTE, TextureImage->pixels);
   break;
   case 1: TextureImage=IMG_Load(F2);
           if (TextureImage==NULL) return(0);
           // INFO: identifikator a nazov suboru textury
           fprintf(stderr,"CM textura: %i %s\n",t_a,F2);
           // vytvorenie objektu textury
           glTexImage2D(GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_ARB,0,GL_RGB,
                        TextureImage->w,TextureImage->h,0,
                        GL_BGR,GL_UNSIGNED_BYTE, TextureImage->pixels);
   break;
   case 2: TextureImage=IMG_Load(F3);
           if (TextureImage==NULL) return(0);
           // INFO: identifikator a nazov suboru textury
           fprintf(stderr,"CM textura: %i %s\n",t_a,F3);
           // vytvorenie objektu textury
           glTexImage2D(GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_ARB,0,GL_RGB,
                        TextureImage->w,TextureImage->h,0,
                        GL_BGR,GL_UNSIGNED_BYTE, TextureImage->pixels);
   break;
   case 3: TextureImage=IMG_Load(F4);
           if (TextureImage==NULL) return(0);
           // INFO: identifikator a nazov suboru textury
           fprintf(stderr,"CM textura: %i %s\n",t_a,F4);
           // vytvorenie objektu textury
           glTexImage2D(GL_TEXTURE_CUBE_MAP_POSITIVE_X_ARB,0,GL_RGB,
                        TextureImage->w,TextureImage->h,0,
                        GL_BGR,GL_UNSIGNED_BYTE, TextureImage->pixels);
   break;
   case 4: TextureImage=IMG_Load(F5);
           if (TextureImage==NULL) return(0);
           // INFO: identifikator a nazov suboru textury
           fprintf(stderr,"CM textura: %i %s\n",t_a,F5);
           // vytvorenie objektu textury
           glTexImage2D(GL_TEXTURE_CUBE_MAP_POSITIVE_Y_ARB,0,GL_RGB,
                        TextureImage->w,TextureImage->h,0,
                        GL_BGR,GL_UNSIGNED_BYTE, TextureImage->pixels);
   break;
   case 5: TextureImage=IMG_Load(F6);
           if (TextureImage==NULL) return(0);
           // INFO: identifikator a nazov suboru textury
           fprintf(stderr,"CM textura: %i %s\n",t_a,F6);
           // vytvorenie objektu textury
           glTexImage2D(GL_TEXTURE_CUBE_MAP_POSITIVE_Z_ARB,0,GL_RGB,
                        TextureImage->w,TextureImage->h,0,
                        GL_BGR,GL_UNSIGNED_BYTE, TextureImage->pixels);
   break;
  }
 }

 // uvolnenie pouzitej pamate
 if (TextureImage) SDL_FreeSurface(TextureImage);
 TextureImage=NULL;

 return(0);
}

// zrusenie OpenGL textury *************************************************
void mTexture::FreeTexture(GLuint ID)
{
 glDeleteTextures(1,&texture_id[ID]);
}

// nahratie detailnej textury terenu ***************************************
int mTexture::LoadDetailTexture(char *F)
{
 // SDL nacitanie suborov s texturami
 SDL_Surface *TextureImage;

 // nahratie suboru s obrazkom
 if ((TextureImage=IMG_Load(F))) {
  // vygenerovanie textury
  glGenTextures(1,&detail_tex_id);

  // INFO: identifikator a nazov suboru textury
  fprintf(stderr,"Detail textura: %i %s\n",detail_tex_id,F);

  // pripojenie textury
  glBindTexture(GL_TEXTURE_2D,detail_tex_id);
  glPixelStorei(GL_UNPACK_ALIGNMENT,1);
  // nastavenie filteringu
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
  // nastavenie opakovania sa textury
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
  // vytvorenie objektu textury
  gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB,TextureImage->w,TextureImage->h,
                    GL_BGR,GL_UNSIGNED_BYTE,TextureImage->pixels);
 }
 else return(1);

 // uvolnenie pouzitej pamate
 if (TextureImage) SDL_FreeSurface(TextureImage);
 TextureImage=NULL;

 return(0);
}

// nahratie textury zameriavacieho kriza ***********************************
int mTexture::LoadCrosshairTexture(char *F)
{
 // SDL nacitanie suborov s texturami
 SDL_Surface *TextureImage;

 // nahratie suboru s obrazkom
 if ((TextureImage=IMG_Load(F))) {
  // vygenerovanie textury
  glGenTextures(1,&crosshair_id);

  // INFO: identifikator a nazov suboru textury
  fprintf(stderr,"Crosshair textura: %i %s\n",crosshair_id,F);

  // pripojenie textury
  glBindTexture(GL_TEXTURE_2D,crosshair_id);
  // nastavenie filteringu
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
  // nastavenie opakovania sa textury
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP);
  // vytvorenie objektu textury
  glTexImage2D(GL_TEXTURE_2D,0,3,TextureImage->w,TextureImage->h,
               0,GL_BGR,GL_UNSIGNED_BYTE,TextureImage->pixels);
 }
 else return(1);

 // uvolnenie pouzitej pamate
 if (TextureImage) SDL_FreeSurface(TextureImage);
 TextureImage=NULL;

 return(0);
}

// vykreslenie zameriavacieho kriza ****************************************
int mTexture::DrawCrosshair(void)
{
 // ulozenie potrebnych atributov
 glPushAttrib(GL_LIGHTING_BIT|GL_DEPTH_BUFFER_BIT|GL_TEXTURE_BIT);
 glDisable(GL_LIGHTING);   // vypnutie osvetlenia
 glDisable(GL_DEPTH_TEST); // vypnutie testu hlbky
 glDepthMask(GL_FALSE);
 glEnable(GL_TEXTURE_2D);  // zapnutie texturovania

 glBlendFunc(GL_ONE,GL_ONE); // zapnutie priesvitnosti
 glEnable(GL_BLEND);

 glPushMatrix();   // modelview
 glLoadIdentity();

 // pripojenie textury zameriavacieho kriza
 glBindTexture(GL_TEXTURE_2D,crosshair_id);
 // zameriavaci kriz na stred okna
 glTranslatef(0.0f,0.0f,-0.1f);
 // stvoruholnik
 glBegin(GL_QUADS);
  glTexCoord2f(0.0f,0.0f); glVertex3f(-0.006f,-0.006f,0.0f);
  glTexCoord2f(1.0f,0.0f); glVertex3f( 0.006f,-0.006f,0.0f);
  glTexCoord2f(1.0f,1.0f); glVertex3f( 0.006f, 0.006f,0.0f);
  glTexCoord2f(0.0f,1.0f); glVertex3f(-0.006f, 0.006f,0.0f);
 glEnd();

 glPopMatrix(); // modelview

 glDisable(GL_BLEND); // vypnutie priesvitnosti

 glPopAttrib(); // vyber ulozenych atributov

 return(0);
}
