// hlavickove subory *******************************************************
// systemove funkcie
#include <math.h>
// GL kniznica
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glext.h>

// vlastny .h subor
#include "mBackend.h"

// konstruktor *************************************************************
mObject::mObject()
{
 // matica 128x128 vrcholov + texturovacich suradnic
 mat128x128=NULL;
}

// destruktor **************************************************************
mObject::~mObject()
{

}

// rovinna plocha **********************************************************
void mObject::plane(GLfloat SIZE,GLfloat STEP)
{
 c=SIZE/2.0f; // polovica velkosti rovinnej plochy

 // vykresli sa (size/step)*(size/step)*2 trojuholnikov
 // napr. pre size=100, step=10 sa vykresli 200 trojuholnikov
 for (b=SIZE;b>0.0f;b-=STEP) {  // Z-rozmer
  for (a=0.0f;a<SIZE;a+=STEP) { // X-rozmer
   glBegin(GL_TRIANGLES);
    glNormal3f(0.0f,1.0f,0.0f); glVertex3f(-c+a,     0.0f,-c+b);
    glNormal3f(0.0f,1.0f,0.0f); glVertex3f(-c+a+STEP,0.0f,-c+b);
    glNormal3f(0.0f,1.0f,0.0f); glVertex3f(-c+a,     0.0f,-c+b-STEP);

    glNormal3f(0.0f,1.0f,0.0f); glVertex3f(-c+a+STEP,0.0f,-c+b);
    glNormal3f(0.0f,1.0f,0.0f); glVertex3f(-c+a+STEP,0.0f,-c+b-STEP);
    glNormal3f(0.0f,1.0f,0.0f); glVertex3f(-c+a,     0.0f,-c+b-STEP);
   glEnd();
  }
 }
}

// vyber udajov o 3ds modely a ich presun do zoznamu trojuholnikov *********
int mObject::extract_terrain(m3ds *OBJ,mTriangle *T,GLuint DTEX,GLuint OL)
{
 // kopia vrcholov trojuholnikov + texturovacich suradnic do matice 128x128
 // z povodnych 129x129 vrcholov sa v procese nacitania 3ds suboru
 // vytvori 128x2x128 trojuholnikov = 32768 faces
 // teren bol vytvoreny v aplikaciach L3DT, Terragen (povodny rozmer 129x129 vrcholov),
 // do 3ds suboru bol exportovany pomocou aplikacie ac3d
 //
 // novym usporiadanim vznikne:
 // 127 trojuholnikov v smere X
 // 127 trojuholnikov v smere Y
 // pre 64 trojuholnikov (j+=4) v smere X sa vezmu 2 vrcholy
 // posledne prave a posledne vrchne trojuholniky budu vynechane
 // vytvori sa matica 128x128 vrcholov

 // pre vsetky objekty ulozene v 3ds subore
 for (obj_id=0;obj_id<OBJ->m3dsModel.ObjectsNum;obj_id++) {

  // ak v 3ds subore nie je ulozeny ani jeden objekt
  if (OBJ->m3dsModel.pObject.size()<=0) return(1);

  // v 3ds subore je ulozeny aspon jeden objekt
  OBJ->TmpObject=&OBJ->m3dsModel.pObject[obj_id];

  // INFO: identifikator a nazov objektu
  //fprintf(stderr,"obj_id: %i name: %s\n",obj_id,OBJ->TmpObject->Name);

  // iba pre objekt "terrain"
  if (!strcmp(OBJ->TmpObject->Name,"terrain")) {

   // vyhradenie pamate pre maticu 128x128
   mat128x128=(tMatVT *) malloc((128*128)*sizeof(tMatVT));
   if (mat128x128==NULL) {
    fprintf(stderr,"Chyba vyhradenia pamate pre maticu 128x128!\n");
    return(1);
   }
   // vynulovanie matice 128x128
   for (i=0;i<(128*128);i++) {
    (mat128x128+i)->s=0;
    (mat128x128+i)->x=0.0f; (mat128x128+i)->y=0.0f; (mat128x128+i)->z=0.0f;
    (mat128x128+i)->u=0.0f; (mat128x128+i)->v=0.0f;
   }
   // 1. prvok matice
   ai=0;

   // pre vsetky steny (face)
   for (j=0;j<OBJ->TmpObject->FacesNum;j+=4) {
    // index na 1. vrchol trojuholnika
    // .
    // . .
    // .   .
    // +......
    i1=OBJ->TmpObject->pFaces[j].vIndex[1];
    // index na 2. vrchol trojuholnika
    // .
    // . .
    // .   .
    // ......+
    i2=OBJ->TmpObject->pFaces[j].vIndex[0];

    // objekt by mal mat texturu
    if (OBJ->TmpObject->isTex) {
     if (OBJ->TmpObject->pTexVerts) {
      // kopia texturovacich suradnic
      (mat128x128+ai)->u  = OBJ->TmpObject->pTexVerts[i1].x;
      (mat128x128+ai)->v  =-OBJ->TmpObject->pTexVerts[i1].y;
      (mat128x128+ai+1)->u= OBJ->TmpObject->pTexVerts[i2].x;
      (mat128x128+ai+1)->v=-OBJ->TmpObject->pTexVerts[i2].y;
     }
    }
    // kopia suradnic vrcholov
    (mat128x128+ai)->x  = OBJ->TmpObject->pVerts[i1].x/1.0f;
    (mat128x128+ai)->y  = OBJ->TmpObject->pVerts[i1].z/1.0f;
    (mat128x128+ai)->z  =-OBJ->TmpObject->pVerts[i1].y/1.0f;
    (mat128x128+ai+1)->x= OBJ->TmpObject->pVerts[i2].x/1.0f;
    (mat128x128+ai+1)->y= OBJ->TmpObject->pVerts[i2].z/1.0f;
    (mat128x128+ai+1)->z=-OBJ->TmpObject->pVerts[i2].y/1.0f;

    // skopiruje sa vzdy dvojica vrcholov
    ai+=2;
   }

   // INFO: pocet skopirovanych vrcholov
   fprintf(stderr,"ai: %i\n",ai);

   // kopia identifikatorov textur
   id1=0; id2=0;
   // objekt by mal mat texturu
   if (OBJ->TmpObject->isTex) {
    id1=OBJ->TextureID[OBJ->TmpObject->texID]; // 1. textura
    id2=DTEX;                                  // 2. textura
   }
   // nastavenie farby
   col.x=1.0f; col.y=1.0f; col.z=1.0f; col.w=1.0f;
   // nastavenie normal
   n1.x=0.0f; n1.y=1.0f; n1.z=0.0f;
   n2.x=0.0f; n2.y=1.0f; n2.z=0.0f;
   n3.x=0.0f; n3.y=1.0f; n3.z=0.0f;

   // optimalizacia matice 128x128
   mat128x128_optimize(OL);

   // vlozenie trojuholnikov do zoznamu trojuholnikov
   // matica 128x128 moze/nemusi byt optimalizovana
   for (ol=2;ol<=OL*2;ol*=2) {
    // v smere Y
    for (bi=0;bi<(128-ol);bi+=ol) {
     // v smere X
     for (ai=0;ai<(128-ol);ai+=ol) {
      // vyber prvku matice 128x128
      ci=ai+(bi*128);

      // korekcia koncov matice z dovodu zabranenia
      // neziaduceho odstranenia niektorych casti matice
      // posledne prave a posledne vrchne trojuholniky budu
      // vlozene do zoznamu bez optimalizacie
      if ((ai==128-(2*ol))) (mat128x128+ci+128*ol/2+ol/2)->s=0;
      if ((bi==128-(2*ol))) (mat128x128+ci+128*ol/2+ol/2)->s=0;

      // ak dana cast matice nebola optimalizovana
      // vykreslia sa trojuholniky patriace tejto casti
      if ((mat128x128+ci+128*ol/2+ol/2)->s==0) {
       // casti matice sa nastavi priznak 100
       (mat128x128+ci+128*ol/2+ol/2)->s=100;

       // vykresli sa 8 alebo 12 trojuholnikov
       i=0;
       while (i<6) {
        switch (i) {
         case 0: di=ci; break;                 // lava dolna dvojica
         case 1: di=ci+ol/2; break;            // prava dolna dvojica
         case 2: di=ci+128*ol/2+ol/2; break;   // prava vrchna dvojica
         case 3: di=ci+128*ol/2; break;        // lava vrchna dvojica
         case 4: di=ci+2*ol/2; break;          // druha prava dolna dvojica
         case 5: di=ci+128*ol/2+2*ol/2; break; // druha prava vrchna dvojica
         default: break;
        }
        i++;

        if (ol==2) { // ak je uroven optimalizacie 2
         // ak je postup v smere X na vrchole 124, vykresli
         // sa druha prava dolna a druha prava vrchna dvojica
         if ((ai<124)&&(i==4)) i=6;
        }
        else if (i==4) i=6; // ak je uroven optimalizacie ina ako 2 vykreslia sa vzdy
                            // iba 4 dvojice

        // ak je uroven optimalizacie viac ako 2 a cast matice uz bola vykreslena,
        // nedojde k zaradeniu trojuholnikov do zoznamu
        if ((ol>2)&&(mat128x128+di+128*ol/4+ol/4)->s==100) { }
        else { // v opacnom pripade sa do zoznamu zaradi dvojica trojuholnikov
         // vrcholy
         v1.x=(mat128x128+di)->x;
         v1.y=(mat128x128+di)->y;
         v1.z=(mat128x128+di)->z;
         v2.x=(mat128x128+di+128*ol/2)->x;
         v2.y=(mat128x128+di+128*ol/2)->y;
         v2.z=(mat128x128+di+128*ol/2)->z;
         v3.x=(mat128x128+di+ol/2)->x;
         v3.y=(mat128x128+di+ol/2)->y;
         v3.z=(mat128x128+di+ol/2)->z;
         // texturovacie suradnice
         t1.x=(mat128x128+di)->u;          t1.y=(mat128x128+di)->v;
         t2.x=(mat128x128+di+128*ol/2)->u; t2.y=(mat128x128+di+128*ol/2)->v;
         t3.x=(mat128x128+di+ol/2)->u;     t3.y=(mat128x128+di+ol/2)->v;
         // pridanie noveho trojuholniku do zoznamu:
         // vrcholy, normaly, texturovacie suradnice, farba, identifikatory dvoch texur
         T->tri_add(v1,v2,v3,n1,n2,n3,t1,t2,t3,col,id1,id2);
         // vrcholy
         v1.x=(mat128x128+di+ol/2)->x;
         v1.y=(mat128x128+di+ol/2)->y;
         v1.z=(mat128x128+di+ol/2)->z;
         v2.x=(mat128x128+di+128*ol/2)->x;
         v2.y=(mat128x128+di+128*ol/2)->y;
         v2.z=(mat128x128+di+128*ol/2)->z;
         v3.x=(mat128x128+di+128*ol/2+ol/2)->x;
         v3.y=(mat128x128+di+128*ol/2+ol/2)->y;
         v3.z=(mat128x128+di+128*ol/2+ol/2)->z;
         // texturovacie suradnice
         t1.x=(mat128x128+di+ol/2)->u;          t1.y=(mat128x128+di+ol/2)->v;
         t2.x=(mat128x128+di+128*ol/2)->u;      t2.y=(mat128x128+di+128*ol/2)->v;
         t3.x=(mat128x128+di+128*ol/2+ol/2)->u; t3.y=(mat128x128+di+128*ol/2+ol/2)->v;
         // pridanie noveho trojuholniku do zoznamu:
         // vrcholy, normaly, texturovacie suradnice, farba, identifikatory dvoch texur
         T->tri_add(v1,v2,v3,n1,n2,n3,t1,t2,t3,col,id1,id2);
        } // else
       } // while
      } // if
     } // for
    } // for
   } // for

   // zrusenie docasnej matice 128x128
   if (mat128x128!=NULL) free(mat128x128);

  } // iba pre objekt terrain

 } // pre vsetky objekty ulozene v 3ds subore

 return(0);
}

// optimalizacia matice 128x128 ********************************************
int mObject::mat128x128_optimize(GLuint OL)
{
 // uroven optimalizacie moze byt 1, 2, 4, 8, 16...
 // 1=ziadna optimalizacia
 // optimalizuje sa z nizsej urovne na vyssiu
 for (ol=2;ol<=OL;ol*=2) {
  // optimalizacia 8 trojuholnikov -> 2 trojuholniky
  // v smere Y
  for (bi=0;bi<(128-ol);bi+=ol) {
   // v smere X
   for (ai=0;ai<(128-ol);ai+=ol) {
    // vyber prvku matice 128x128
    ci=ai+(bi*128);

    // ak je uroven optimalizacie vyssia ako 2
    // dojde k spocitaniu priznakov nastavenych
    // v predoslom kroku optimalizacie
    if (ol>2) {
     i=(mat128x128+ci+128*ol/4+ol/4)->s+
       (mat128x128+ci+128*ol/4+ol/4+ol/2)->s+
       (mat128x128+ci+128*ol/4+ol/4+128*ol/2)->s+
       (mat128x128+ci+128*ol/4+ol/4+128*ol/2+ol/2)->s;

     // INFO: pocet optimalizovanych stvoric
     //fprintf(stderr,"s: %i\n",i);
    }
    // ak je uroven optimalizacie vyssia ako 2
    // a sucet priznakov nie je 4, nebude sa prevadzat
    // dalsia optimalizacia
    if ((ol>2)&&(i!=4)) { }
    // v opacnom pripade dojde k optimalizacii
    // predtym optimalizovanej casti matice
    else {
     // 4 krajne vrcholy stvorcovej casti matice
     v1.x=(mat128x128+ci)->x;
     v1.y=(mat128x128+ci)->y;
     v1.z=(mat128x128+ci)->z;
     v2.x=(mat128x128+ci+ol)->x;
     v2.y=(mat128x128+ci+ol)->y;
     v2.z=(mat128x128+ci+ol)->z;
     v3.x=(mat128x128+ci+128*ol+ol)->x;
     v3.y=(mat128x128+ci+128*ol+ol)->y;
     v3.z=(mat128x128+ci+128*ol+ol)->z;
     v4.x=(mat128x128+ci+128*ol)->x;
     v4.y=(mat128x128+ci+128*ol)->y;
     v4.z=(mat128x128+ci+128*ol)->z;
     // stredny vrchol
     v5.x=(mat128x128+ci+128*ol/2+ol/2)->x;
     v5.y=(mat128x128+ci+128*ol/2+ol/2)->y;
     v5.z=(mat128x128+ci+128*ol/2+ol/2)->z;

     // vypocet odchylky na hlavnej diagonale
     p1=mm.Normalize(mm.Vector(v2,v4));
     p2=mm.Normalize(mm.Vector(v2,v5));

     // INFO: uhol medzi vektormi
     //fprintf(stderr,"angle: %f\n",mm.Dot(p1,p2));

     if (fabs(mm.Dot(p1,p2))>0.995f) {
      // vypocet odchylky na vedlajsej diagonale
      p1=mm.Normalize(mm.Vector(v1,v3));
      p2=mm.Normalize(mm.Vector(v1,v5));
      // v pripade, ze stvorcove casti matice tvoria
      // "takmer" rovinnu plochu, budu sa optimalizovat
      if (fabs(mm.Dot(p1,p2))>0.995f) (mat128x128+ci+128*ol/2+ol/2)->s=1;
     }
    }
   }
  }
 }

 // uroven optimalizacie moze byt 1, 2, 4, 8, 16...
 // nove suradnice vrcholov sa vypocitavaju postupom
 // z vyssej urovne na nizsiu
 for (ol=OL;ol>=2;ol/=2) {
  // v smere Y
  for (bi=0;bi<(128-ol);bi+=ol) {
   // v smere X
   for (ai=0;ai<(128-ol);ai+=ol) {
    // vyber prvku matice 128x128
    ci=ai+(bi*128);

    // ak bola dana stvorcova cast matice optimalizovana
    // je potrebne prepocitat suradnice stredneho vrcholu a
    // vrcholov na jej hranach a to na zaklade krajnych vrcholov
    // novo vytvorenej stvorcovej casti
    if ((mat128x128+ci+128*ol/2+ol/2)->s==1) {
     // 4 krajne vrcholy stvorcovej casti matice
     v1.x=(mat128x128+ci)->x;
     v1.y=(mat128x128+ci)->y;
     v1.z=(mat128x128+ci)->z;
     v2.x=(mat128x128+ci+ol)->x;
     v2.y=(mat128x128+ci+ol)->y;
     v2.z=(mat128x128+ci+ol)->z;
     v3.x=(mat128x128+ci+128*ol+ol)->x;
     v3.y=(mat128x128+ci+128*ol+ol)->y;
     v3.z=(mat128x128+ci+128*ol+ol)->z;
     v4.x=(mat128x128+ci+128*ol)->x;
     v4.y=(mat128x128+ci+128*ol)->y;
     v4.z=(mat128x128+ci+128*ol)->z;
     // hlavna diagonala stvorcovej casti matice
     p1=mm.Vector(v2,v4); p1=mm.DivVectorByScalar(p1,2.0f);
     // vypocet novej suradnice stredneho vrcholu
     (mat128x128+ci+128*ol/2+ol/2)->x=v2.x+p1.x;
     (mat128x128+ci+128*ol/2+ol/2)->y=v2.y+p1.y;
     (mat128x128+ci+128*ol/2+ol/2)->z=v2.z+p1.z;
     // hrany stvorcovej casti matice
     p1=mm.Vector(v1,v2); p1=mm.DivVectorByScalar(p1,(GLfloat) ol);
     p2=mm.Vector(v4,v3); p2=mm.DivVectorByScalar(p2,(GLfloat) ol);
     p3=mm.Vector(v1,v4); p3=mm.DivVectorByScalar(p3,(GLfloat) ol);
     p4=mm.Vector(v2,v3); p4=mm.DivVectorByScalar(p4,(GLfloat) ol);
     // vypocet novych suradnic vrcholov patriacich
     // hranam v smere X
     for (i=1;i<ol;i++) {
      (mat128x128+ci+i)->x=v1.x+p1.x*(GLfloat) i;
      (mat128x128+ci+i)->y=v1.y+p1.y*(GLfloat) i;
      (mat128x128+ci+i)->z=v1.z+p1.z*(GLfloat) i;
      (mat128x128+ci+i+128*ol)->x=v4.x+p2.x*(GLfloat) i;
      (mat128x128+ci+i+128*ol)->y=v4.y+p2.y*(GLfloat) i;
      (mat128x128+ci+i+128*ol)->z=v4.z+p2.z*(GLfloat) i;
     }
     // vypocet novych suradnic vrcholov patriacich
     // hranam v smere Y
     for (i=1;i<ol;i++) {
      (mat128x128+ci+128*i)->x=v1.x+p3.x*(GLfloat) i;
      (mat128x128+ci+128*i)->y=v1.y+p3.y*(GLfloat) i;
      (mat128x128+ci+128*i)->z=v1.z+p3.z*(GLfloat) i;
      (mat128x128+ci+128*i+ol)->x=v2.x+p4.x*(GLfloat) i;
      (mat128x128+ci+128*i+ol)->y=v2.y+p4.y*(GLfloat) i;
      (mat128x128+ci+128*i+ol)->z=v2.z+p4.z*(GLfloat) i;
     }
    }
   }
  }
 }

 return(0);
}

// vykreslenie matice 128x128 **********************************************
int mObject::mat128x128_draw(void)
{
 // ulozenie potrebnych atributov
 glPushAttrib(GL_LIGHTING_BIT|GL_TEXTURE_BIT);
 glDisable(GL_LIGHTING);   // vypnutie osvetlenia
 glDisable(GL_TEXTURE_2D); // vypnutie texturovania

 // body
 glBegin(GL_POINTS);
 // trojuholniky
 //glBegin(GL_TRIANGLES);

  for (bi=0;bi<127;bi++) {
   for (ai=0;ai<127;ai++) {
    ci=ai+(bi*128);
    // body
    glVertex3f((mat128x128+ci)->x,(mat128x128+ci)->y,(mat128x128+ci)->z);
    /*
    // trojuholniky
    glVertex3f((mat128x128+ci)->x,(mat128x128+ci)->y,(mat128x128+ci)->z);
    glVertex3f((mat128x128+ci+1)->x,(mat128x128+ci+1)->y,(mat128x128+ci+1)->z);
    glVertex3f((mat128x128+ci+128)->x,(mat128x128+ci+128)->y,(mat128x128+ci+128)->z);
    glVertex3f((mat128x128+ci+1)->x,(mat128x128+ci+1)->y,(mat128x128+ci+1)->z);
    glVertex3f((mat128x128+ci+129)->x,(mat128x128+ci+129)->y,(mat128x128+ci+129)->z);
    glVertex3f((mat128x128+ci+128)->x,(mat128x128+ci+128)->y,(mat128x128+ci+128)->z);
    */
   }
  }

 glEnd();

 // vyber ulozenych atributov - LIGHTING, TEXTURE
 glPopAttrib();

 return(0);
}

// vyber udajov o 3ds dummy objektoch **************************************
int mObject::extract_dummy(m3ds *OBJ)
{
 // pre vsetky objekty ulozene v 3ds subore
 for (obj_id=0;obj_id<OBJ->m3dsModel.ObjectsNum;obj_id++) {

  // ak v 3ds subore nie je ulozeny ani jeden objekt
  if (OBJ->m3dsModel.pObject.size()<=0) return(1);

  // v 3ds subore je ulozeny aspon jeden objekt
  OBJ->TmpObject=&OBJ->m3dsModel.pObject[obj_id];

  // INFO: identifikator a nazov objektu
  //fprintf(stderr,"obj_id: %i name: %s\n",obj_id,OBJ->TmpObject->Name);

  // iba pre objekt "avatar"
  if (!strcmp(OBJ->TmpObject->Name,"avatar")) {
   i1=OBJ->TmpObject->pFaces[0].vIndex[0];
   avatar_pos.x=-OBJ->TmpObject->pVerts[i1].x/1.0f;
   avatar_pos.y=-OBJ->TmpObject->pVerts[i1].z/1.0f;
   avatar_pos.z= OBJ->TmpObject->pVerts[i1].y/1.0f;
  } // iba pre objekt avatar

 } // pre vsetky objekty ulozene v 3ds subore

 return(0);
}

// vyber udajov o 3ds modeloch a ich presun do zoznamu trojuholnikov *******
int mObject::extract_entities(m3ds *OBJ,mTriangle *T)
{
 // pre vsetky objekty ulozene v 3ds subore
 for (obj_id=0;obj_id<OBJ->m3dsModel.ObjectsNum;obj_id++) {

  // ak v 3ds subore nie je ulozeny ani jeden objekt
  if (OBJ->m3dsModel.pObject.size()<=0) return(1);

  // v 3ds subore je ulozeny aspon jeden objekt
  OBJ->TmpObject=&OBJ->m3dsModel.pObject[obj_id];

  // iba pre objekty "rect"
  if (strstr(OBJ->TmpObject->Name,"rect")) {

   // nastavenie farby
   col.x=1.0f; col.y=1.0f; col.z=1.0f; col.w=1.0f;
   // kopia identifikatorov textur
   id1=0; id2=0;
   // objekt by mal mat texturu
   if (OBJ->TmpObject->isTex) {
    id1=OBJ->TextureID[OBJ->TmpObject->texID]; // 1. textura
    id2=0;                                     // 2. textura
   }
   // nastavenie normal
   n1.x=0.0f; n1.y=1.0f; n1.z=0.0f;
   n2.x=0.0f; n2.y=1.0f; n2.z=0.0f;
   n3.x=0.0f; n3.y=1.0f; n3.z=0.0f;

   // INFO: identifikator, nazov objektu, identifikator textury
   //fprintf(stderr,"obj_id: %i name: %s texID: %i\n",obj_id,OBJ->TmpObject->Name,OBJ->TmpObject->texID);

   // pre vsetky steny (face)
   for (j=0;j<OBJ->TmpObject->FacesNum;j++) {
    i1=OBJ->TmpObject->pFaces[j].vIndex[0];
    i2=OBJ->TmpObject->pFaces[j].vIndex[1];
    i3=OBJ->TmpObject->pFaces[j].vIndex[2];

    // objekt by mal mat texturu
    if (OBJ->TmpObject->isTex) {
     if (OBJ->TmpObject->pTexVerts) {
      // kopia texturovacich suradnic
      t1.x= OBJ->TmpObject->pTexVerts[i1].x;
      t1.y=-OBJ->TmpObject->pTexVerts[i1].y;
      t2.x= OBJ->TmpObject->pTexVerts[i2].x;
      t2.y=-OBJ->TmpObject->pTexVerts[i2].y;
      t3.x= OBJ->TmpObject->pTexVerts[i3].x;
      t3.y=-OBJ->TmpObject->pTexVerts[i3].y;
     }
    }
    // kopia suradnic vrcholov
    v1.x= OBJ->TmpObject->pVerts[i1].x;
    v1.y= OBJ->TmpObject->pVerts[i1].z;
    v1.z=-OBJ->TmpObject->pVerts[i1].y;
    v2.x= OBJ->TmpObject->pVerts[i2].x;
    v2.y= OBJ->TmpObject->pVerts[i2].z;
    v2.z=-OBJ->TmpObject->pVerts[i2].y;
    v3.x= OBJ->TmpObject->pVerts[i3].x;
    v3.y= OBJ->TmpObject->pVerts[i3].z;
    v3.z=-OBJ->TmpObject->pVerts[i3].y;

    // INFO: suradnice vrcholov
    //fprintf(stderr,"%2.1f %2.1f %2.1f %2.1f %2.1f %2.1f %2.1f %2.1f %2.1f\n",
    //               v1.x,v1.y,v1.z,v2.x,v2.y,v2.z,v3.x,v3.y,v3.z);

    // pridanie noveho trojuholniku do zoznamu:
    // vrcholy, normaly, texturovacie suradnice, farba, identifikatory dvoch texur
    T->tri_add(v1,v2,v3,n1,n2,n3,t1,t2,t3,col,id1,id2);
   }

  } // iba pre objekty rect

 } // pre vsetky objekty ulozene v 3ds subore

 return(0);
}
