// hlavickove subory *******************************************************
// systemove funkcie
#include <stdio.h>
#include <stdlib.h>
// GL kniznica
#include <GL/gl.h>
#include <GL/glext.h>

// vlastny .h subor
#include "mFrontend.h"

// konstruktor *************************************************************
mTriangle::mTriangle()
{
 // nastavenie pociatocnych hodnot jednotlivym premennym
 first_tri=NULL; last_tri=NULL;
 tmp_tri=NULL;
 // celkovy pocet trojuholnikov
 tri_num=0;

 // pole aktivnych trojuholnikov
 act_tri=NULL;
 // pocet aktivnych trojuholnikov
 act_tri_num=0;

 // identifikator aktualnej textury
 act_tex_id=0;
}

// destruktor **************************************************************
mTriangle::~mTriangle()
{
 // vymazanie zoznamu trojuholnikov
 tri_delall();

 // vymazanie pola aktivnych trojuholnikov
 if (act_tri!=NULL) { free(act_tri); act_tri=NULL; act_tri_num=0; }
}

// pridanie noveho trojuholnika ********************************************
int mTriangle::tri_add(tVec3 V1,tVec3 V2,tVec3 V3,
                       tVec3 N1,tVec3 N2,tVec3 N3,
                       tVec2 T1,tVec2 T2,tVec2 T3,
                       tVec4 C,
                       GLuint TEX1,GLuint TEX2)
{
 if (first_tri==NULL) { // vytvorenie 1. trojuholnika
  first_tri=new _tri;
  if (first_tri==NULL) {
   fprintf(stderr,"Chyba vyhradenia pamate pre novy trojuholnik!\n");
   return(1);
  }
  last_tri=first_tri;
 }
 else {                 // vytvorenie dalsieho trojuholnika
  last_tri->next=new _tri;
  if (last_tri->next==NULL) {
   fprintf(stderr,"Chyba vyhradenia pamate pre novy trojuholnik!\n");
   return(1);
  }
  last_tri=last_tri->next;
 }

 // identifikator trojuholnika
 last_tri->id=tri_num;

 tri_num++; // zvysenie celkoveho poctu trojuholnikov

 // vrcholy trojuholnika
 last_tri->v1x=V1.x; last_tri->v1y=V1.y; last_tri->v1z=V1.z;
 last_tri->v2x=V2.x; last_tri->v2y=V2.y; last_tri->v2z=V2.z;
 last_tri->v3x=V3.x; last_tri->v3y=V3.y; last_tri->v3z=V3.z;
 // normaly vo vrcholoch
 last_tri->n1x=N1.x; last_tri->n1y=N1.y; last_tri->n1z=N1.z;
 last_tri->n2x=N2.x; last_tri->n2y=N2.y; last_tri->n2z=N2.z;
 last_tri->n3x=N3.x; last_tri->n3y=N3.y; last_tri->n3z=N3.z;
 // texturovacie suradnice
 last_tri->t1x=T1.x; last_tri->t1y=T1.y;
 last_tri->t2x=T2.x; last_tri->t2y=T2.y;
 last_tri->t3x=T3.x; last_tri->t3y=T3.y;
 // farba
 last_tri->c1=C.x; last_tri->c2=C.y; last_tri->c3=C.z; last_tri->c4=C.w;
 // identifikatory textur
 last_tri->tex1id=TEX1;
 last_tri->tex2id=TEX2;
 // minimalne a maximalne suradnice vrcholov na danych osiach
 last_tri->minx=last_tri->v1x;
 if (last_tri->v2x<last_tri->minx) last_tri->minx=last_tri->v2x;
 if (last_tri->v3x<last_tri->minx) last_tri->minx=last_tri->v3x;
 last_tri->maxx=last_tri->v1x;
 if (last_tri->v2x>last_tri->maxx) last_tri->maxx=last_tri->v2x;
 if (last_tri->v3x>last_tri->maxx) last_tri->maxx=last_tri->v3x;

 last_tri->miny=last_tri->v1y;
 if (last_tri->v2y<last_tri->miny) last_tri->miny=last_tri->v2y;
 if (last_tri->v3y<last_tri->miny) last_tri->miny=last_tri->v3y;
 last_tri->maxy=last_tri->v1y;
 if (last_tri->v2y>last_tri->maxy) last_tri->maxy=last_tri->v2y;
 if (last_tri->v3y>last_tri->maxy) last_tri->maxy=last_tri->v3y;

 last_tri->minz=last_tri->v1z;
 if (last_tri->v2z<last_tri->minz) last_tri->minz=last_tri->v2z;
 if (last_tri->v3z<last_tri->minz) last_tri->minz=last_tri->v3z;
 last_tri->maxz=last_tri->v1z;
 if (last_tri->v2z>last_tri->maxz) last_tri->maxz=last_tri->v2z;
 if (last_tri->v3z>last_tri->maxz) last_tri->maxz=last_tri->v3z;
 // indikator spracovania
 last_tri->finish_bit=false;
 // odkaz na nasledujuci trojuholnik
 last_tri->next=NULL;

 return(0);
}

// priprava pola pre aktivne trojuholniky **********************************
int mTriangle::tri_finish(GLuint N)
{
 // priprava pola takej velkosti, aby don bolo mozne ulozit
 // vsetky trojuholniky pouziteho stromu, avsak z tohto pola bude pouzita
 // iba cast, ktora sa naplni odkazmi na aktivne trojuholniky
 act_tri=(_tri **) malloc(N*sizeof(_tri));
 if (act_tri==NULL) {
  fprintf(stderr,"Chyba vyhradenia pamate pre aktivne trojuholniky!\n");
  return(1);
 }

 return(0);
}

// vykreslenie vsetkych trojuholnikov ulozenych v zozname trojuholnikov ****
int mTriangle::tri_draw(mFrustum *F)
{
 // ulozenie potrebnych atributov
 glPushAttrib(GL_LIGHTING_BIT|GL_TEXTURE_BIT);
 glDisable(GL_LIGHTING); // vypnutie osvetlenia

 // prvy trojuholnik v zozname
 tmp_tri=first_tri; if (tmp_tri==NULL) return(1);

 // identifikator aktualnej textury
 // k vymene textur dojde iba v pripade, ak je to potrebne,
 // inak zostane aktivna povodna textura
 act_tex_id=0;

 // 1. texturovacia jednotka
 glActiveTextureARB(GL_TEXTURE0_ARB);
 glEnable(GL_TEXTURE_2D);
 // 2. texturovacia jednotka
 glActiveTextureARB(GL_TEXTURE1_ARB);
 glEnable(GL_TEXTURE_2D);
 // kombinacna funkcia
 glTexEnvi(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_COMBINE_EXT);
 glTexEnvi(GL_TEXTURE_ENV,GL_COMBINE_RGB_EXT,GL_ADD_SIGNED_EXT);

 // pre vsetky trojuholniky v zozname
 while (tmp_tri!=NULL) {

  // Frustum Culling
  if (F->CubeInFrustum2(tmp_tri->minx,tmp_tri->maxx,
                        tmp_tri->miny,tmp_tri->maxy,
                        tmp_tri->minz,tmp_tri->maxz)) {

   // ak je potrebne vymenit aktualnu texturu
   if (tmp_tri->tex1id!=act_tex_id) {
    // identifikator novej aktualnej textury
    act_tex_id=tmp_tri->tex1id;
    // pouzije sa multitexturing
    // 1. textura - original
    // 2. textura - pre teren to bude detailna textura
    //              pre ostatne objekty moze zostat neurcena
    glActiveTextureARB(GL_TEXTURE0_ARB);
    glBindTexture(GL_TEXTURE_2D,tmp_tri->tex1id);
    glActiveTextureARB(GL_TEXTURE1_ARB);
    glBindTexture(GL_TEXTURE_2D,tmp_tri->tex2id);
   }

   // vykreslenie trojuholnikov
   glBegin(GL_TRIANGLES);
    // farba
    //glColor4f(tmp_tri->c1,tmp_tri->c2,tmp_tri->c3,tmp_tri->c4);
    // normala, texturovacie suradnice, vrchol
    glNormal3f(tmp_tri->n1x,tmp_tri->n1y,tmp_tri->n1z);
    glMultiTexCoord2fARB(GL_TEXTURE0_ARB,tmp_tri->t1x,tmp_tri->t1y);
    glMultiTexCoord2fARB(GL_TEXTURE1_ARB,200.0f*tmp_tri->t1x,200.0f*tmp_tri->t1y);
    //glTexCoord2f(tmp_tri->t1x,tmp_tri->t1y); // standardne texturovanie
    glVertex3f(tmp_tri->v1x,tmp_tri->v1y,tmp_tri->v1z);
    // normala, texturovacie suradnice, vrchol
    glNormal3f(tmp_tri->n2x,tmp_tri->n2y,tmp_tri->n2z);
    glMultiTexCoord2fARB(GL_TEXTURE0_ARB,tmp_tri->t2x,tmp_tri->t2y);
    glMultiTexCoord2fARB(GL_TEXTURE1_ARB,200.0f*tmp_tri->t2x,200.0f*tmp_tri->t2y);
    //glTexCoord2f(tmp_tri->t2x,tmp_tri->t2y); // standardne texturovanie
    glVertex3f(tmp_tri->v2x,tmp_tri->v2y,tmp_tri->v2z);
    // normala, texturovacie suradnice, vrchol
    glNormal3f(tmp_tri->n3x,tmp_tri->n3y,tmp_tri->n3z);
    glMultiTexCoord2fARB(GL_TEXTURE0_ARB,tmp_tri->t3x,tmp_tri->t3y);
    glMultiTexCoord2fARB(GL_TEXTURE1_ARB,200.0f*tmp_tri->t3x,200.0f*tmp_tri->t3y);
    //glTexCoord2f(tmp_tri->t3x,tmp_tri->t3y); // standardne texturovanie
    glVertex3f(tmp_tri->v3x,tmp_tri->v3y,tmp_tri->v3z);
   glEnd();

  } // Frustum Culling

  // vyber dalsieho trojuholnika zo zoznamu
  tmp_tri=tmp_tri->next;

 } // pre vsetky trojuholniky v zozname

 // 2. texturovacia jednotka
 glActiveTextureARB(GL_TEXTURE1_ARB);
 glDisable(GL_TEXTURE_2D);
 // 1. texturovacia jednotka
 glActiveTextureARB(GL_TEXTURE0_ARB);

 // vyber ulozenych atributov - LIGHTING, TEXTURE
 glPopAttrib();

 // nastavenie standardnej farby - biela
 //glColor4f(1.0f,1.0f,1.0f,1.0f);

 return(0);
}

// vykreslenie aktivnych trojuholnikov *************************************
int mTriangle::act_tri_draw(mFrustum *F)
{
 // ulozenie potrebnych atributov
 glPushAttrib(GL_LIGHTING_BIT|GL_TEXTURE_BIT);
 glDisable(GL_LIGHTING); // vypnutie osvetlenia

 // identifikator aktualnej textury
 // k vymene textur dojde iba v pripade, ak je to potrebne,
 // inak zostane aktivna povodna textura
 act_tex_id=0;

 // 1. texturovacia jednotka
 glActiveTextureARB(GL_TEXTURE0_ARB);
 glEnable(GL_TEXTURE_2D);
 // 2. texturovacia jednotka
 glActiveTextureARB(GL_TEXTURE1_ARB);
 glEnable(GL_TEXTURE_2D);
 // kombinacna funkcia
 glTexEnvi(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_COMBINE_EXT);
 glTexEnvi(GL_TEXTURE_ENV,GL_COMBINE_RGB_EXT,GL_ADD_SIGNED_EXT);

 render_tri=0;

 // pre vsetky trojuholniky v poli
 for (a=0;a<act_tri_num;a++) {

  // ak sa v poli nachadza rovnaky trojuholnik viackrat (z dovodu
  // pouzitia stromu), spracuje a zobrazi sa vzdy iba jeden krat
  if (!act_tri[a]->finish_bit) {
   act_tri[a]->finish_bit=true;

   // Frustum Culling
   if (F->CubeInFrustum2(act_tri[a]->minx,act_tri[a]->maxx,
                         act_tri[a]->miny,act_tri[a]->maxy,
                         act_tri[a]->minz,act_tri[a]->maxz)) {

    // ak je potrebne vymenit aktualnu texturu
    if (act_tri[a]->tex1id!=act_tex_id) {
     // identifikator novej aktualnej textury
     act_tex_id=act_tri[a]->tex1id;
     // pouzije sa multitexturing
     // 1. textura - original
     // 2. textura - pre teren to bude detailna textura
     //              pre ostatne objekty moze zostat neurcena
     glActiveTextureARB(GL_TEXTURE0_ARB);
     glBindTexture(GL_TEXTURE_2D,act_tri[a]->tex1id);
     glActiveTextureARB(GL_TEXTURE1_ARB);
     glBindTexture(GL_TEXTURE_2D,act_tri[a]->tex2id);
    }

    render_tri++;

    // vykreslenie trojuholnikov
    glBegin(GL_TRIANGLES);
     // farba
     //glColor4f(act_tri[a]->c1,act_tri[a]->c2,act_tri[a]->c3,act_tri[a]->c4);
     // normala, texturovacie suradnice, vrchol
     glNormal3f(act_tri[a]->n1x,act_tri[a]->n1y,act_tri[a]->n1z);
     glMultiTexCoord2fARB(GL_TEXTURE0_ARB,act_tri[a]->t1x,act_tri[a]->t1y);
     glMultiTexCoord2fARB(GL_TEXTURE1_ARB,200.0f*act_tri[a]->t1x,200.0f*act_tri[a]->t1y);
     //glTexCoord2f(act_tri[a]->t1x,act_tri[a]->t1y); // standardne texturovanie
     glVertex3f(act_tri[a]->v1x,act_tri[a]->v1y,act_tri[a]->v1z);
     // normala, texturovacie suradnice, vrchol
     glNormal3f(act_tri[a]->n2x,act_tri[a]->n2y,act_tri[a]->n2z);
     glMultiTexCoord2fARB(GL_TEXTURE0_ARB,act_tri[a]->t2x,act_tri[a]->t2y);
     glMultiTexCoord2fARB(GL_TEXTURE1_ARB,200.0f*act_tri[a]->t2x,200.0f*act_tri[a]->t2y);
     //glTexCoord2f(act_tri[a]->t2x,act_tri[a]->t2y); // standardne texturovanie
     glVertex3f(act_tri[a]->v2x,act_tri[a]->v2y,act_tri[a]->v2z);
     // normala, texturovacie suradnice, vrchol
     glNormal3f(act_tri[a]->n3x,act_tri[a]->n3y,act_tri[a]->n3z);
     glMultiTexCoord2fARB(GL_TEXTURE0_ARB,act_tri[a]->t3x,act_tri[a]->t3y);
     glMultiTexCoord2fARB(GL_TEXTURE1_ARB,200.0f*act_tri[a]->t3x,200.0f*act_tri[a]->t3y);
     //glTexCoord2f(act_tri[a]->t3x,act_tri[a]->t3y); // standardne texturovanie
     glVertex3f(act_tri[a]->v3x,act_tri[a]->v3y,act_tri[a]->v3z);
    glEnd();

   } // Frustum Culling

  } // finish_bit

 } // pre vsetky trojuholniky v poli

 // 2. texturovacia jednotka
 glActiveTextureARB(GL_TEXTURE1_ARB);
 glDisable(GL_TEXTURE_2D);
 // 1. texturovacia jednotka
 glActiveTextureARB(GL_TEXTURE0_ARB);

 // vyber ulozenych atributov - LIGHTING, TEXTURE
 glPopAttrib();

 // nastavenie standardnej farby - biela
 //glColor4f(1.0f,1.0f,1.0f,1.0f);

 return(0);
}

// vymazanie zoznamu trojuholnikov *****************************************
int mTriangle::tri_delall(void)
{
 if (first_tri!=NULL) {
  while (first_tri->next!=NULL) { // zmazanie vsetkych trojuholnikov zo zoznamu
   tmp_tri=first_tri->next;
   delete first_tri;
   first_tri=tmp_tri;
  }
  delete first_tri;               // zmazanie 1. trojuholnika
  first_tri=NULL;
 }
 last_tri=NULL; tmp_tri=NULL;

 tri_num=0; // celkovy pocet trojuholnikov

 return(0);
}
