#ifndef _MFRONTEND_H_
#define _MFRONTEND_H_

// matematika
#include "common/m3dmath.h"

// Frustum Culling
#include "common/mFrustum.h"

// struktura udajov o trojuholniku
typedef struct __tri {
 GLuint id; // identifikator trojuholnika

 GLfloat v1x,v1y,v1z; // 1. vrchol
 GLfloat v2x,v2y,v2z; // 2. vrchol
 GLfloat v3x,v3y,v3z; // 3. vrchol

 GLfloat n1x,n1y,n1z; // 1. normala
 GLfloat n2x,n2y,n2z; // 2. normala
 GLfloat n3x,n3y,n3z; // 3. normala

 GLfloat t1x,t1y; // 1. texturovacie suradnice
 GLfloat t2x,t2y; // 2. texturovacie suradnice
 GLfloat t3x,t3y; // 3. texturovacie suradnice

 GLfloat c1,c2,c3,c4; // farba

 GLuint tex1id; // identifikator 1. textury
 GLuint tex2id; // identifikator 2. textury

 // minimalne a maximalne suradnice vrcholov na danych osiach
 // okolo trojuholnika sa tak vytvori pomyselny hranol
 GLfloat minx,maxx,miny,maxy,minz,maxz;

 // indikator spracovania, ak finish_bit=1 trojuholnik
 // bol spracovany a nebude sa dalej spracovavat
 bool finish_bit;

 __tri *next; // odkaz na nasledujuci trojuholnik
} _tri;

// pracuje sa s dvoma objektami obsahujucimi trojuholniky:
//
// 1. objekt je zoznam, na ktoreho prvy prvok odkazuje
// premenna first_tri, v tomto zozname su ulozene vsetky
// trojuholniky
//
// 2. objekt je pole aktivnych trojuholnikov, ktore obsahuje
// odkazy na tie trojuholniky z predosleho zoznamu, ktore
// sa maju spracovat a zobrazit na obrazovke, ostatne
// trojuholniky nie su z urciteho dovodu (prechadzanie stromu,
// Frustum Culling) viditelne a nebudu spracovavane

// trieda premennych a funkcii (objektov) pre pracu s OpenGL trojuholnikmi
class mTriangle {
public:
 mTriangle();          // konstruktor
 virtual ~mTriangle(); // destruktor

 _tri *first_tri,*last_tri; // prvy, posledny trojuholnik v zozname
 _tri *tmp_tri;             // pomocna premenna
 GLuint tri_num;            // celkovy pocet trojuholnikov

 _tri **act_tri;     // pole odkazov na aktivne trojuholniky
 GLuint act_tri_num; // pocet aktivnych trojuholnikov
 GLuint render_tri;  // pocet zobrazenych trojuholnikov

 // pridanie noveho trojuholnika
 int tri_add(tVec3 V1,tVec3 V2,tVec3 V3,
             tVec3 N1,tVec3 N2,tVec3 N3,
             tVec2 T1,tVec2 T2,tVec2 T3,
             tVec4 C,
             GLuint TEX1,GLuint TEX2);
 // priprava pola odkazov na aktivne trojuholniky
 // je potrebne vykonat po pridani posledneho trojuholnika
 // do zoznamu, dalsie trojuholniky sa uz nebudu pridavat
 int tri_finish(GLuint N);

 // vykreslenie vsetkych trojuholnikov ulozenych v zozname trojuholnikov
 int tri_draw(mFrustum *F);
 // vykreslenie aktivnych trojuholnikov
 int act_tri_draw(mFrustum *F);

 // vymazanie zoznamu trojuholnikov
 int tri_delall(void);
private:
 GLuint a; // pomocna premenna

 GLuint act_tex_id; // identifikator aktualnej textury
};

#endif
