// hlavickove subory *******************************************************
// GL kniznica
#include <GL/gl.h>
// funkcie SDL kniznice
#include <SDL/SDL.h>

// vlastny .h subor
#include "mInterface.h"

// konstruktor *************************************************************
mInterface::mInterface()
{
 // rozmery okna
 ViewPort[0]=0; ViewPort[1]=0; ViewPort[2]=0; ViewPort[3]=0;

 lighting_onoff=0;  // osvetlenie
 help_onoff=0;      // napoveda
 pause_motion=0;    // casovo zavisly pohyb
 normals_onoff=0;   // vypnutie vykreslovania normal
 polygon_mode=0;    // vyplnene trojuholniky
 lod=2;             // uroven detailov

 // identifikator pouziteho AVATARa
 Aid=0;
 // pozicia vypisu 1. textu pocas startu aplikacie
 StartInfoPos=10.0f;

 // instancia triedy mObject - praca s objektami
 ogl_Object=NULL;
}

// destruktor **************************************************************
mInterface::~mInterface()
{
 // zrusenie instancie triedy mObject - praca s objektami
 if (ogl_Object!=NULL) { delete ogl_Object; ogl_Object=NULL; }
}

// vypis info pocas startu aplikacie ***************************************
void mInterface::PrintStartInfo(char *I)
{
 // vypis formatovaneho textu
 ogl_Text.p_ftext(10.0f,StartInfoPos,"%s",I);

 // vymena zobrazovacich zasobnikov, zobrazenie textu na obrazovke
 SDL_GL_SwapBuffers();

 // dalsi text bude vypisany na novu poziciu
 StartInfoPos+=30.0f;
 // pauza aby bol text aspon chvilku viditelny
 SDL_Delay(200);
}

// inicializacia OpenGL - nastavenie parametrov pre tvorbu OpenGL sceny ****
int mInterface::ogl_Initialize(void)
{
 // nastavenie 2 vseobecnych materialov
 for (t_a=0;t_a<2;t_a++) {
  ogl_Material[t_a].amb[0]=0.1f; ogl_Material[t_a].amb[1]=0.1f;
  ogl_Material[t_a].amb[2]=0.1f; ogl_Material[t_a].amb[3]=1.0f;
  ogl_Material[t_a].dif[0]=0.8f; ogl_Material[t_a].dif[1]=0.8f;
  ogl_Material[t_a].dif[2]=0.8f; ogl_Material[t_a].dif[3]=1.0f;
  ogl_Material[t_a].spe[0]=1.0f; ogl_Material[t_a].spe[1]=1.0f;
  ogl_Material[t_a].spe[2]=1.0f; ogl_Material[t_a].spe[3]=1.0f;
  ogl_Material[t_a].amb_dif[0]=0.25f; ogl_Material[t_a].amb_dif[1]=0.25f;
  ogl_Material[t_a].amb_dif[2]=0.25f; ogl_Material[t_a].amb_dif[3]=1.0f;
  ogl_Material[t_a].emi[0]=0.0f; ogl_Material[t_a].emi[1]=0.0f;
  ogl_Material[t_a].emi[2]=0.0f; ogl_Material[t_a].emi[3]=1.0f;
  ogl_Material[t_a].shi=128.0f;
  ogl_Material[t_a].transp=0.0f;
  ogl_Material[t_a].sillum[0]=0.0f; ogl_Material[t_a].sillum[1]=0.0f;
  ogl_Material[t_a].sillum[2]=0.0f; ogl_Material[t_a].sillum[3]=1.0f;
 }
 // nastavenie 2 vseobecnych svetiel
 for (t_a=0;t_a<2;t_a++) {
  ogl_Light[t_a].amb[0]=0.1f; ogl_Light[t_a].amb[1]=0.1f;
  ogl_Light[t_a].amb[2]=0.1f; ogl_Light[t_a].amb[3]=1.0f;
  ogl_Light[t_a].dif[0]=0.9f; ogl_Light[t_a].dif[1]=0.9f;
  ogl_Light[t_a].dif[2]=0.9f; ogl_Light[t_a].dif[3]=1.0f;
  ogl_Light[t_a].spe[0]=0.9f; ogl_Light[t_a].spe[1]=0.9f;
  ogl_Light[t_a].spe[2]=0.9f; ogl_Light[t_a].spe[3]=1.0f;
  ogl_Light[t_a].pos[0]=50.0f; ogl_Light[t_a].pos[1]=100.0f;
  ogl_Light[t_a].pos[2]=50.0f; ogl_Light[t_a].pos[3]=1.0f;
  // pos[3]=1.0f - vsesmerove svetlo, 0.0f - smerove svetlo
  ogl_Light[t_a].spot[0]=0.0f;
  ogl_Light[t_a].spot[1]=0.0f;
  ogl_Light[t_a].spot[2]=-1.0f;
 }

 // STAVOVY STROJ STAVOVY STROJ STAVOVY STROJ STAVOVY STROJ STAVOVY STROJ --

 // nastavenie OpenGL zasobnika farieb a hlbky *****************************
 glClearColor(0.5f,0.6f,0.7f,0.5f); // zasobnik farieb bude vymazavany ciernou farbou
 glClearDepth(1.0f);                // zasobnik hlbky bude vymazavany nastavenim na 1.0f

 glDepthFunc(GL_LEQUAL);  // typ porovnavacej funkcie pre urcenie Z-vzdialenosti
 glEnable(GL_DEPTH_TEST); // zapnutie testovania hlbky

 glDisable(GL_CULL_FACE); // nastavenie poradia, v ktorom budu definovane vrcholy objektu
 glFrontFace(GL_CCW);     // reprezentovat privratene resp. odvratene plochy,
 glCullFace(GL_BACK);     // CCW - proti smeru hod. ruciciek, CW - v smere hod. ruciciek
 glEnable(GL_CULL_FACE);  // odvratene plochy sa nebudu spracovavat

 // vseobecny material objektov
 glMaterialfv(GL_FRONT,GL_AMBIENT,ogl_Material[0].amb);
 glMaterialfv(GL_FRONT,GL_DIFFUSE,ogl_Material[0].dif);
 glMaterialfv(GL_FRONT,GL_SPECULAR,ogl_Material[0].spe);
 glMaterialfv(GL_FRONT,GL_EMISSION,ogl_Material[0].emi);
 glMaterialf(GL_FRONT,GL_SHININESS,ogl_Material[0].shi);
 // farba bude mat vplyv iba na 'diffuse' komponent materialu
 glColorMaterial(GL_FRONT,GL_DIFFUSE);
 // pre Gourandove svetelne vypocty bude pouzity material priradeny objektu,
 // ktory bude ale stale ovplyvneny farbou zadavanou prikazom glColor
 glEnable(GL_COLOR_MATERIAL);

 // zapnutie gourand-jemneho tienovania
 glShadeModel(GL_SMOOTH);
 // trojuholniky budu vyplnene
 glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
 // zapnutie modulacie textur osvetlenim
 // vysledna farba bodu = farba z textury + osvetlenie
 glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);
 // realnejsie perspektivne vypocty a korekcie
 glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);

 // nastavenie parametrov 1. OpenGL svetla
 glLightfv(GL_LIGHT0,GL_AMBIENT,ogl_Light[0].amb);
 glLightfv(GL_LIGHT0,GL_DIFFUSE,ogl_Light[0].dif);
 glLightfv(GL_LIGHT0,GL_SPECULAR,ogl_Light[0].spe);
 glLightfv(GL_LIGHT0,GL_POSITION,ogl_Light[0].pos);
 glEnable(GL_LIGHT0); // ON
 // nastavenie parametrov 2. OpenGL svetla
 glLightfv(GL_LIGHT1,GL_AMBIENT,ogl_Light[1].amb);
 glLightfv(GL_LIGHT1,GL_DIFFUSE,ogl_Light[1].dif);
 glLightfv(GL_LIGHT1,GL_SPECULAR,ogl_Light[1].spe);
 glLightfv(GL_LIGHT1,GL_POSITION,ogl_Light[1].pos);
 //glEnable(GL_LIGHT1); // OFF
 if (lighting_onoff) glEnable(GL_LIGHTING);
 else glDisable(GL_LIGHTING);
 // svetlo sa pocita len pre privratene strany
 glLightModeli(GL_LIGHT_MODEL_TWO_SIDE,0);
 // svetelne vypocty su lokalne
 glLightModelf(GL_LIGHT_MODEL_LOCAL_VIEWER,1.0f);

 // nastavenie standardnej farby - biela
 glColor4f(1.0f,1.0f,1.0f,1.0f);

 // INICIALIZACIA SCENY INICIALIZACIA SCENY INICIALIZACIA SCENY ------------

 // vymazanie zasobnika farieb a hlbky
 glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

 fprintf(stderr,"----------------------------------------------------------\n");

 // inicializacia fontu
 if (ogl_Text.t_initialize("data/font/font.png",1800,1215)) {
  fprintf(stderr,"Chyba inicializacie fontu!\n"); return(1);
 }
 // nastavenie farby textu
 ogl_Text.t_color(0.8f,0.8f,0.0f,1.0f);

 // potvrdenie startu OpenGL
 PrintStartInfo("Nastavenie OpenGL stavoveho stroja...");
 PrintStartInfo("OK");

 // test rozsireni
 PrintStartInfo("Test podporovanych rozsireni...");
 if (CheckExtension("GL_ARB_multitexture")) {
  fprintf(stderr,"Nie su podporovane potrebne rozsirenia!\n"); return(1);
 }
 PrintStartInfo("OK");

 // zameriavaci kriz
 PrintStartInfo("Nahratie textury zameriavacieho kriza...");
 if (ogl_Texture.LoadCrosshairTexture("data/gfx/crosshair1.bmp")) {
  fprintf(stderr,"Chyba nahratia textury zameriavacieho kriza!\n"); return(1);
 }
 PrintStartInfo("OK");

 // nahratie detailnej textury - specialne pre teren
 PrintStartInfo("Nahratie detailnej textury pre teren...");
 if (ogl_Texture.LoadDetailTexture("data/textures/detail.bmp")) {
  fprintf(stderr,"Chyba nahratia detailnej textury pre teren!\n"); return(1);
 }
 PrintStartInfo("OK");

 // vytvorenie instancie triedy mObject - praca s objektami
 ogl_Object=new mObject;
 if (ogl_Object==NULL) {
  fprintf(stderr,"Chyba vytvorenia instancie triedy mObject!\n"); return(1);
 }

 // nacitanie 3ds suboru
 PrintStartInfo("Nahratie 3ds suboru...");
 if (ogl_Object->m3ds_obj.imp3ds("data/model/terrain.3ds")) {
  fprintf(stderr,"Chyba nacitania 3ds suboru!\n"); return(1);
 }
 PrintStartInfo("OK");

 // extrahovanie trojuholnikov z 3ds suboru pre stanovenu uroven detailov
 PrintStartInfo("Extrahovanie trojuholnikov z nacitaneho 3ds suboru...");
 if (ogl_Object->extract_terrain(&ogl_Object->m3ds_obj,
                                 &ogl_Terrain,
                                 ogl_Texture.detail_tex_id,lod)) {
  fprintf(stderr,"Chyba extrahovania trojuholnikov z nacitaneho 3ds suboru!\n"); return(1);
 }
 PrintStartInfo("OK");

 // vytvorenie oktanoveho stromu
 PrintStartInfo("Vytvorenie oktanoveho stromu...");
 if (ogl_Octtree.Octtree_create(&ogl_Terrain,ogl_Terrain.tri_num)) {
  fprintf(stderr,"Chyba vytvorenia oktanoveho stromu!\n"); return(1);
 }
 PrintStartInfo("OK");

 // priprava pola pre aktivne trojuholniky
 ogl_Terrain.tri_finish(ogl_Octtree.octtree_tri_num);

 // extrahovanie dummy objektov z 3ds suboru
 PrintStartInfo("Extrahovanie dummy objektov z nacitaneho 3ds suboru...");
 if (ogl_Object->extract_dummy(&ogl_Object->m3ds_obj)) {
  fprintf(stderr,"Chyba extrahovania dummy objektov z nacitaneho 3ds suboru!\n"); return(1);
 }
 PrintStartInfo("OK");

 Avatar[Aid].posx=ogl_Object->avatar_pos.x;
 Avatar[Aid].posy=ogl_Object->avatar_pos.y;
 Avatar[Aid].posz=ogl_Object->avatar_pos.z;

 // zrusenie instancie triedy mObject - praca s objektami
 if (ogl_Object!=NULL) { delete ogl_Object; ogl_Object=NULL; }

 // vytvorenie instancie triedy mObject - praca s objektami
 ogl_Object=new mObject;
 if (ogl_Object==NULL) {
  fprintf(stderr,"Chyba vytvorenia instancie triedy mObject!\n"); return(1);
 }

 // nacitanie 3ds suboru
 PrintStartInfo("Nahratie 3ds suboru...");
 if (ogl_Object->m3ds_obj.imp3ds("data/model/cube.3ds")) {
  fprintf(stderr,"Chyba nacitania 3ds suboru!\n"); return(1);
 }
 PrintStartInfo("OK");

 // extrahovanie entit z 3ds suboru
 PrintStartInfo("Extrahovanie entit z nacitaneho 3ds suboru...");
 if (ogl_Object->extract_entities(&ogl_Object->m3ds_obj,&ogl_Entities)) {
  fprintf(stderr,"Chyba extrahovania entit z nacitaneho 3ds suboru!\n"); return(1);
 }
 PrintStartInfo("OK");

 // zrusenie instancie triedy mObject - praca s objektami
 if (ogl_Object!=NULL) { delete ogl_Object; ogl_Object=NULL; }

 // koniec pociatocnych nastaveni, zaciatok interakcie pozorovatela
 PrintStartInfo("START");

 return(0);
}

// vykreslenie OpenGL sceny ************************************************
int mInterface::ogl_Draw(void)
{
 // vymazanie zasobnika farieb a hlbky
 glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

 // zresetovanie modelovej matice
 glLoadIdentity();

 // TRANSFORMACIA SCENY PODLA NASTAVENI AKTUALNEJ KAMERY!
 glMultMatrixf(Avatar[Aid].matrix);
 // TRANSFORMACIA SCENY PODLA NASTAVENI AKTUALNEJ KAMERY!
 glTranslatef(Avatar[Aid].posx,Avatar[Aid].posy,Avatar[Aid].posz);

 // vypocet rovnic stran (rovin) Frustum - vykonava sa vzdy
 ogl_Frustum.FrustumUpdate();

 //glRotatef(45.0f,1.0f,0.0f,0.0f);  // zapnutie specialneho pohladu na scenu
 //glRotatef(-30.0f,0.0f,1.0f,0.0f);
 //glTranslatef(-7.5f,-10.0f,-15.0f);

 // transformacia polohy svetla podla polohy kamery
 ogl_Light[0].pos[0]=-Avatar[Aid].posx;
 ogl_Light[0].pos[1]=-Avatar[Aid].posy;
 ogl_Light[0].pos[2]=-Avatar[Aid].posz;

 // umiestenenie svetla na preddefinovanu poziciu
 glLightfv(GL_LIGHT0,GL_POSITION,ogl_Light[0].pos);

 // vyber trojuholnikov z oktanoveho stromu a presun odkazov do triedy mTriangle
 ogl_Octtree.Octtree_finish(&ogl_Terrain,&ogl_Frustum,-Avatar[Aid].posx,-Avatar[Aid].posy,-Avatar[Aid].posz);
 // vykreslenie aktivnych trojuholnikov terenu
 ogl_Terrain.act_tri_draw(&ogl_Frustum);

 // vykreslenie trojuholnikov entit
 //ogl_Entities.tri_draw(&ogl_Frustum);

 // vykreslenie zameriavacieho kriza
 ogl_Texture.DrawCrosshair();

 // vykreslenie napovedy
 if (help_onoff) {

 }
 else {
  // vypis FPS
  ogl_Text.p_ftext(10.0f,1180.0f,"fps: %.0f",Avatar[Aid].cam_fps);
  // info o polohe AVATARa
  ogl_Text.p_ftext(10.0f,1080.0f,"x: %.1f",-Avatar[Aid].posx);
  ogl_Text.p_ftext(10.0f,1110.0f,"y: %.1f",-Avatar[Aid].posy);
  ogl_Text.p_ftext(10.0f,1140.0f,"z: %.1f",-Avatar[Aid].posz);
  // info o pocte trojuholnikov terenu
  ogl_Text.p_ftext(10.0f,10.0f,"teren: %i",ogl_Terrain.tri_num);
  ogl_Text.p_ftext(10.0f,40.0f,"zobrazenych: %i",ogl_Terrain.render_tri);
  ogl_Text.p_ftext(10.0f,100.0f,"pocet testov: %i",ogl_Octtree.tests_num);
 }

 return(0);
}

// casovo zavisla uprava objektov, resp. sceny *****************************
int mInterface::ogl_Iddle(void)
{
 if (pause_motion==0) {

 }

 return(0);
}

// test podpory rozsirenia grafickou kartou ********************************
int mInterface::CheckExtension(const char *EXT_NAME)
{
 static char *extensions=NULL;

 // ziskanie retazca s podporovanymi rozsireniami grafickej karty
 if (!extensions) extensions=(char *) glGetString(GL_EXTENSIONS);

 // test, ci sa v retazci nachadza pozadovane rozsirenie
 if (!strstr(extensions,EXT_NAME)) {
  fprintf(stderr,"OpenGL rozsirenie %s nie podporovane!\n",EXT_NAME);
  return(1);
 }
 return(0);
}

// vyplnene trojuholniky/iba hrany *****************************************
void mInterface::change_polygon_mode(void)
{
 if (polygon_mode==0) {
  polygon_mode=1;
  // budu vykreslene hrany trojuholnikov
  glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
 }
 else {
  polygon_mode=0;
  // trojuholniky budu vyplnene
  glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
 }

 // pauza medzi stlaceniami klaves
 SDL_Delay(200);
}

// zmena urovne detailov ***************************************************
int mInterface::change_LOD(void)
{
 if (ogl_Object!=NULL) {
  // zmena urovne detailov 1, 2, 4, 8, 16...
  if (lod<16) lod*=2; else lod=1;
  // vymazanie vsetkych trojuholnikov objektu teren
  ogl_Terrain.tri_delall();
  // extrahovanie trojuholnikov z 3ds suboru pre stanovenu uroven detailov
  if (ogl_Object->extract_terrain(&ogl_Object->m3ds_obj,
                                  &ogl_Terrain,
                                  ogl_Texture.detail_tex_id,lod)) {
   fprintf(stderr,"Chyba extrahovania trojuholnikov z nacitaneho 3ds suboru!\n"); return(1);
  }
 }

 // pauza medzi stlaceniami klaves
 SDL_Delay(200);

 return(0);
}
