#ifndef _MINTERFACE_H_
#define _MINTERFACE_H_

// triedy pre pracu s jednotlivymi modulmi Enginu
#include "mAvatar.h"
#include "common/mText.h"
#include "common/mTexture.h"
#include "mBackend.h"
#include "common/mOcttree.h"

typedef struct _material { // struktura OpenGL materialu
 GLfloat amb[4],dif[4],spe[4],amb_dif[4],emi[4];
 GLfloat shi,transp,sillum[4];
};

typedef struct _light { // struktura OpenGL svetla
 GLfloat amb[4],dif[4],spe[4];
 GLfloat pos[4],spot[3];
};

// trieda premennych a funkcii (objektov) pre pracu s OpenGL
class mInterface {
public:
 mInterface();          // konstruktor
 virtual ~mInterface(); // destruktor

 GLint ViewPort[4]; // rozmery okna

 mAvatar Avatar[2]; // 2 Avatari

 // zmena stavov OpenGL stavoveho stroja -----------------------------------
 GLubyte lighting_onoff;  // zap/vyp osvetlenia
 GLubyte help_onoff;      // zap/vyp vypisu helpu
 GLubyte pause_motion;    // zap/vyp casovo zavisleho pohybu objektov
 GLubyte normals_onoff;   // zap/vyp vykreslovania normal
 GLubyte polygon_mode;    // vyplnene trojuholniky/iba hrany
 GLubyte lod;             // uroven detailov 1, 2, 4, 8, 16...

 int ogl_Initialize(void); // inicializacia OpenGL
 int ogl_Draw(void);       // vykreslenie OpenGL sceny
 int ogl_Iddle(void);      // casovo zavisla uprava objektov, resp. sceny

 void change_polygon_mode(void); // vyplnene trojuholniky/iba hrany
 int change_LOD(void);           // zmena urovne detailov
private:
 GLuint Aid; // identifikator pouziteho Avatara

 _material ogl_Material[2]; // pouzite materialy

 _light ogl_Light[2];       // pouzite svetla

 GLuint t_a,t_b,t_c; // pomocne premenne

 GLint ogl_State[2]; // pre zistovanie stavov OpenGL stavoveho stroja

 mText ogl_Text;        // instancia triedy mText - praca s textom
 mTexture ogl_Texture;  // instancia triedy mTexture - praca s texturou
 mObject *ogl_Object;   // instancia triedy mObject - praca s objektami

 mTriangle ogl_Terrain;  // instancia triedy mTriangle - teren
 mTriangle ogl_Entities; // instancia triedy mTriangle - entity

 mFrustum ogl_Frustum; // instancia triedy mFrustum - Frustum Culling
 mOcttree ogl_Octtree; // instancia triedy mOcttree - oktanovy strom

 GLfloat StartInfoPos;         // pozicia vypisu start info
 void PrintStartInfo(char *I); // vypis info pocas startu aplikacie

 // test podpory rozsirenia grafickou kartou
 int CheckExtension(const char *EXT_NAME);
};

#endif
